@@ -20,6 +20,7 @@
 #include "Cipher.h"
 #include "base64.h"
 
+#include <string.h>
 #include <rlog/rlog.h>
 #include <rlog/Error.h>
 #include <rlog/RLogChannel.h>
@@ -28,18 +29,19 @@
 
 using namespace rlog;
 using namespace rel;
+using namespace boost;
 
 static RLogChannel * Info = DEF_CHANNEL( "info/nameio", Log_Info );
 
 
-static Ptr<NameIO> NewBlockNameIO( const Interface &iface,
-	const Ptr<Cipher> &cipher, const CipherKey &key )
+static shared_ptr<NameIO> NewBlockNameIO( const Interface &iface,
+	const shared_ptr<Cipher> &cipher, const CipherKey &key )
 {
     int blockSize = 8;
-    if(!cipher.isNull())
+    if(cipher)
 	blockSize = cipher->cipherBlockSize();
 
-    return Ptr<NameIO>( new BlockNameIO( iface, cipher, key, blockSize));
+    return shared_ptr<NameIO>( new BlockNameIO( iface, cipher, key, blockSize));
 }
 
 static bool BlockIO_registered = NameIO::Register("Block",
@@ -70,7 +72,7 @@ Interface BlockNameIO::CurrentInterface()
 }
 
 BlockNameIO::BlockNameIO( const rel::Interface &iface,
-	const rel::Ptr<Cipher> &cipher, 
+	const shared_ptr<Cipher> &cipher, 
 	const CipherKey &key, int blockSize )
     : _interface( iface.current() )
     , _bs( blockSize )
@@ -184,9 +186,16 @@ int BlockNameIO::decodeName( const char *encodedName, int length,
 	throw ERROR( "invalid padding size" );
     }
 
+    // copy out the result..
+    memcpy(plaintextName, tmpBuf+2, finalSize);
+    plaintextName[finalSize] = '\0';
+
     // check the mac
     unsigned int mac2 = _cipher->MAC_16((const unsigned char *)tmpBuf+2,
 	    decodedStreamLen, _key, iv);
+    
+    BUFFER_RESET( tmpBuf );
+
     if(mac2 != mac)
     {
 	rDebug("checksum mismatch: expected %u, got %u", mac, mac2);
@@ -194,12 +203,6 @@ int BlockNameIO::decodeName( const char *encodedName, int length,
 	throw ERROR( "checksum mismatch in filename decode" );
     }
 
-    // ok, it is safe to copy out the result..
-    memcpy(plaintextName, tmpBuf+2, finalSize);
-    plaintextName[finalSize] = '\0';
-
-    BUFFER_RESET( tmpBuf );
-
     return finalSize;
 }
 
