@@ -27,8 +27,26 @@
 #include <string>
 #include <iostream>
 
+// for static build.  Need to reference the modules which are registered at
+// run-time, to ensure that the linker doesn't optimize them away.
+#include "NullCipher.h"
+#include "SSL_Cipher.h"
+
 using namespace std;
 using namespace rel;
+using boost::shared_ptr;
+
+#define REF_MODULE(TYPE)  \
+    if( !TYPE::Enabled() ) \
+	cerr << "referenceModule: should never happen\n";
+
+static
+void AddSymbolReferences()
+{
+    REF_MODULE(SSL_Cipher)
+    REF_MODULE(NullCipher)
+}
+
 
 struct CipherAlg
 {
@@ -46,8 +64,10 @@ static CipherMap_t *gCipherMap = NULL;
 std::list<Cipher::CipherAlgorithm> 
 Cipher::GetAlgorithmList( bool includeHidden )
 {
+    AddSymbolReferences();
+
     list<CipherAlgorithm> result;
-    
+
     if(!gCipherMap)
 	return result;
 
@@ -100,9 +120,9 @@ bool Cipher::Register(const char *name, const char *description,
     return true;
 }
 
-Ptr<Cipher> Cipher::New(const string &name, int keyLen)
+shared_ptr<Cipher> Cipher::New(const string &name, int keyLen)
 {
-    Ptr<Cipher> result;
+    shared_ptr<Cipher> result;
 
     if(gCipherMap)
     {
@@ -118,9 +138,9 @@ Ptr<Cipher> Cipher::New(const string &name, int keyLen)
     return result;
 }
 
-Ptr<Cipher> Cipher::New( const Interface &iface, int keyLen )
+shared_ptr<Cipher> Cipher::New( const Interface &iface, int keyLen )
 {
-    Ptr<Cipher> result;
+    shared_ptr<Cipher> result;
     if(gCipherMap)
     {
 	CipherMap_t::const_iterator it;
@@ -186,25 +206,3 @@ bool Cipher::nameDecode( unsigned char *data, int len,
     return streamDecode( data, len, iv64, key );
 }
 
-#ifdef BUILD_STATIC
-
-// for static build.  Need to reference the modules which are registered at
-// run-time, to ensure that the linker doesn't optimize them away.
-#include "CipherV3.h"
-#include "NullCipher.h"
-#include "SSL_Cipher.h"
-
-#define REF_MODULE(TYPE)  \
-    if( !TYPE::Enabled() ) \
-	cerr << "referenceModule: should never happen\n";
-
-static
-void AddSymbolReferences()
-{
-    REF_MODULE(CipherV3)
-    REF_MODULE(SSL_Cipher)
-    REF_MODULE(NullCipher)
-}
-
-#endif
-
