@@ -25,6 +25,8 @@
 #include <fcntl.h>
 #include <errno.h>
 
+using boost::shared_ptr;
+
 /*
     - Version 2:0 adds support for a per-file initialization vector with a
       fixed 8 byte header.  The headers are enabled globally within a
@@ -46,10 +48,10 @@ static bool checkSize( int fsBlockSize, int cipherBlockSize )
 	return false;
 }
 
-CipherFileIO::CipherFileIO( const Ptr<FileIO> &_base, 
-	const Ptr<Cipher> &_cipher,
+CipherFileIO::CipherFileIO( const shared_ptr<FileIO> &_base, 
+	const shared_ptr<Cipher> &_cipher,
 	const CipherKey &_key, int fsBlockSize,
-	bool uniqueIV )
+	bool uniqueIV, bool _reverseEncryption )
     : BlockFileIO( fsBlockSize )
     , base( _base )
     , cipher( _cipher )
@@ -58,8 +60,10 @@ CipherFileIO::CipherFileIO( const Ptr<FileIO> &_base,
     , externalIV( 0 )
     , fileIV( 0 )
     , lastFlags( 0 )
+    , reverseEncryption( _reverseEncryption )
 {
     static bool warnOnce = false;
+
     if(!warnOnce)
 	warnOnce = checkSize( fsBlockSize, cipher->cipherBlockSize() );
 }
@@ -287,12 +291,10 @@ ssize_t CipherFileIO::readOneBlock( const IORequest &req ) const
 
 	if(readSize != bs)
 	{
-	    ok = cipher->streamDecode( req.data, (int)readSize, 
-		    blockNum ^ fileIV, key );
+	    ok = streamRead( req.data, (int)readSize, blockNum ^ fileIV); 
 	} else
 	{
-	    ok = cipher->blockDecode( req.data, (int)readSize, 
-		    blockNum ^ fileIV, key );
+	    ok = blockRead( req.data, (int)readSize, blockNum ^ fileIV); 
 	}
 
 	if(!ok)
@@ -307,6 +309,7 @@ ssize_t CipherFileIO::readOneBlock( const IORequest &req ) const
     return readSize;
 }
 
+
 bool CipherFileIO::writeOneBlock( const IORequest &req )
 {
     int bs = blockSize();
@@ -318,12 +321,12 @@ bool CipherFileIO::writeOneBlock( const IORequest &req )
     bool ok;
     if( req.dataLen != bs )
     {
-	ok = cipher->streamEncode( req.data, (int)req.dataLen, 
-		blockNum ^ fileIV, key );
+	ok = streamWrite( req.data, (int)req.dataLen, 
+		blockNum ^ fileIV );
     } else
     {
-	ok = cipher->blockEncode( req.data, (int)req.dataLen, 
-		blockNum ^ fileIV, key );
+	ok = blockWrite( req.data, (int)req.dataLen, 
+		blockNum ^ fileIV );
     }
 
     if( ok )
@@ -344,6 +347,56 @@ bool CipherFileIO::writeOneBlock( const IORequest &req )
     return ok;
 }
 
+bool CipherFileIO::blockWrite( unsigned char *buf, int size, 
+	             uint64_t _iv64 ) const
+{
+    if (!reverseEncryption)
+	return cipher->blockEncode( buf, size, _iv64, key );
+    else
+	return cipher->blockDecode( buf, size, _iv64, key );
+} 
+
+bool CipherFileIO::streamWrite( unsigned char *buf, int size, 
+	             uint64_t _iv64 ) const
+{
+    if (!reverseEncryption)
+	return cipher->streamEncode( buf, size, _iv64, key );
+    else
+	return cipher->streamDecode( buf, size, _iv64, key );
+} 
+
+
+bool CipherFileIO::blockRead( unsigned char *buf, int size, 
+	             uint64_t _iv64 ) const
+{
+    if (reverseEncryption)
+	return cipher->blockEncode( buf, size, _iv64, key );
+    else
+    {
+        if(_allowHoles)
+        {
+            // special case - leave all 0's alone
+            for(int i=0; i<size; ++i)
+                if(buf[i] != 0)
+                    return cipher->blockDecode( buf, size, _iv64, key );
+
+            return true;
+        } else
+            return cipher->blockDecode( buf, size, _iv64, key );
+    }
+} 
+
+bool CipherFileIO::streamRead( unsigned char *buf, int size, 
+	             uint64_t _iv64 ) const
+{
+    if (reverseEncryption)
+	return cipher->streamEncode( buf, size, _iv64, key );
+    else
+	return cipher->streamDecode( buf, size, _iv64, key );
+} 
+
+
+
 int CipherFileIO::truncate( off_t size )
 {
     int res = 0;
