@@ -19,12 +19,11 @@
 #define _CipherFileIO_incl_
 
 #include "BlockFileIO.h"
-#include "OpaqueValue.h"
+#include "CipherKey.h"
 
 #include <inttypes.h>
 
 class Cipher;
-typedef rel::OpaqueValue CipherKey;
 
 /*
     Implement the FileIO interface encrypting data in blocks. 
@@ -34,10 +33,10 @@ typedef rel::OpaqueValue CipherKey;
 class CipherFileIO : public BlockFileIO
 {
 public:
-    CipherFileIO( const Ptr<FileIO> &base, 
-	    const Ptr<Cipher> &cipher, 
+    CipherFileIO( const boost::shared_ptr<FileIO> &base, 
+	    const boost::shared_ptr<Cipher> &cipher, 
 	    const CipherKey &key, int blockSize,
-	    bool uniqueIV );
+	    bool uniqueIV, bool reverseEncryption );
     virtual ~CipherFileIO();
 
     virtual rel::Interface interface() const;
@@ -61,9 +60,17 @@ private:
 
     void initHeader();
     bool writeHeader();
+    bool blockRead( unsigned char *buf, int size, 
+	             uint64_t iv64 ) const;
+    bool streamRead( unsigned char *buf, int size, 
+	             uint64_t iv64 ) const;
+    bool blockWrite( unsigned char *buf, int size, 
+	             uint64_t iv64 ) const;
+    bool streamWrite( unsigned char *buf, int size, 
+	             uint64_t iv64 ) const;
 
-    Ptr<FileIO> base;
-    Ptr<Cipher> cipher;
+    boost::shared_ptr<FileIO> base;
+    boost::shared_ptr<Cipher> cipher;
     CipherKey key;
     // if haveHeader is true, then we have a transparent file header which
     // contains a 64 bit initialization vector.
@@ -72,6 +79,7 @@ private:
     uint64_t externalIV;
     uint64_t fileIV;
     int lastFlags;
+    bool reverseEncryption;
 };
 
 #endif
