@@ -31,6 +31,7 @@
 #include <sys/fsuid.h>
 #endif
 
+#include "Context.h"
 #include "Cipher.h"
 #include "Mutex.h"
 #include <rlog/rlog.h>
@@ -44,153 +45,52 @@ using namespace rlog;
 
 static RLogChannel *Info = DEF_CHANNEL( "info/DirNode", Log_Info );
 
-// the usefullness of the cache drops dramatically after 1 or 2 nodes..
-const int RecentNodeCacheSize = 3;
-
-#define CACHE_STATS 0
-
-#if CACHE_STATS
-
-static int cacheHits[RecentNodeCacheSize] = {0};
-static int cacheMiss = 0;
-
-#endif
-
-/*
-    If REQUEST_HISTORY is enabled, then we store a bunch of extra data about
-    what happened with a file.  It is only useful for debugging, and probably
-    only for very particular uses..
-*/
-#define REQUEST_HISTORY 0
-
-/*
-    If USE_CIPHERTEXT_KEYS is defined, then internal node data is stored
-    indexed by ciphertext name rather then plaintext name.
-
-    Indexing by plaintext name is faster, but indexing by ciphertext names
-    might reduce the possibility of plaintext names being left in memory..
-
-    Perhaps this can be a runtime option on a per-filesystem basis..
-*/
-#ifdef USE_CIPHERTEXT_KEYS
-#   define HASHKEY(KEY) naming->encodePath(KEY).c_str()
-#else
-#   define HASHKEY(KEY) KEY
-#endif
-
-#define ATIME_ATOMIC_UPDATE \
-if(inactivityTimer) \
-{ \
-    Lock _lock( mutex ); \
-    lastAccess = time(0); \
-}
-
-
-
-#if REQUEST_HISTORY
-/*
-    For debugging purposes.  This keeps track of each request type for every
-    file.  Lots of data..
-*/
-struct RequestHistory
+class DirDeleter
 {
-    list< string > requestor;
+public:
+    void operator () ( DIR *d )
+    {
+	::closedir( d );
+    }
 };
 
-static pthread_mutex_t gMutex = PTHREAD_MUTEX_INITIALIZER;
-map<string, RequestHistory*> gRequestHistory;
 
-static void addHistory( const char *plainName, const char *info )
-{
-    pthread_mutex_lock( &gMutex );
-    map<string,RequestHistory*>::iterator it = gRequestHistory.find(plainName);
-    if(it != gRequestHistory.end())
-    {
-	it->second->requestor.push_back( string( info ) );
-    } else
-    {
-	RequestHistory *rq = new RequestHistory;
-	rq->requestor.push_back( string( info ) );
-	gRequestHistory.insert( make_pair( string(plainName), rq ));
-    }
-    pthread_mutex_unlock( &gMutex );
-}
-
-static
-void showHistory( const char *name )
-{
-    pthread_mutex_lock( &gMutex );
-    map<string, RequestHistory*>::iterator it = gRequestHistory.find( name );
-
-    if(it != gRequestHistory.end())
-    {
-	cout << "request history for filename " << name << ":\n";
-	list<string>::iterator lit = it->second->requestor.begin();
-	for(; lit != it->second->requestor.end(); ++lit)
-	{
-	    cout << "    requested by \"" << (*lit) << "\"\n";
-	}
-    } else
-    {
-	cout << "no history for filename " << name << endl;
-    }
-
-    pthread_mutex_unlock( &gMutex );
-}
-#endif
-
-
-
-DirTraverse::DirTraverse(DIR *_dirPtr, uint64_t _iv, const NameIO *_naming)
-    : LinkedOwner()
-    , dir( _dirPtr )
+DirTraverse::DirTraverse(const shared_ptr<DIR> &_dirPtr, 
+	uint64_t _iv, const shared_ptr<NameIO> &_naming)
+    : dir( _dirPtr )
     , iv( _iv )
     , naming( _naming )
 {
 }
 
 DirTraverse::DirTraverse(const DirTraverse &src)
-    : LinkedOwner()
-    , dir( src.dir )
+    : dir( src.dir )
     , iv( src.iv )
     , naming( src.naming )
 {
-    shareOwnership( &src );
 }
 
 DirTraverse &DirTraverse::operator = (const DirTraverse &src)
 {
-    if(dir != src.dir)
-    {
-	if(dropOwnership() && dir)
-	    ::closedir( (DIR*)dir );
-
-	shareOwnership( &src );
-
-	dir = src.dir;
-	iv = src.iv;
-	naming = src.naming;
-    }
+    dir = src.dir;
+    iv = src.iv;
+    naming = src.naming;
     
     return *this;
 }
 
 DirTraverse::~DirTraverse()
 {
-    if(dropOwnership())
-    {
-	if(dir)
-	    ::closedir( (DIR*)dir );
-	dir = NULL;
-	iv = 0;
-	naming = 0;
-    }
+    dir.reset();
+    iv = 0;
+    naming.reset();
 }
 
 static
-bool _nextName(struct dirent *&de, DIR *dir, int *fileType, ino_t *inode)
+bool _nextName(struct dirent *&de, const shared_ptr<DIR> &dir, 
+	int *fileType, ino_t *inode)
 {
-    de = ::readdir( dir );
+    de = ::readdir( dir.get() );
 
     if(de)
     {
@@ -254,12 +154,143 @@ std::string DirTraverse::nextInvalid()
     return string();
 }
 
-DirNode::DirNode(const string &sourceDir, const Ptr<Config> &_config)
+struct RenameEl
+{
+    // ciphertext names
+    string oldCName;
+    string newCName; // intermediate name (not final cname)
+
+    // plaintext names
+    string oldPName;
+    string newPName;
+
+    bool isDirectory;
+};
+
+class RenameOp
+{
+private:
+    DirNode *dn;
+    shared_ptr< list<RenameEl> > renameList;
+    list<RenameEl>::const_iterator last;
+
+public:
+    RenameOp( DirNode *_dn, const shared_ptr< list<RenameEl> > &_renameList )
+	: dn(_dn), renameList(_renameList)
+    {
+	last = renameList->begin();
+    }
+
+    RenameOp(const RenameOp &src)
+	: dn(src.dn)
+	, renameList(src.renameList)
+	, last(src.last)
+    {
+    }
+
+    ~RenameOp()
+    {
+	if(renameList)
+	{
+	    // got a bunch of decoded filenames sitting in memory..  do a little
+	    // cleanup before leaving..
+	    list<RenameEl>::iterator it;
+	    for(it = renameList->begin(); it != renameList->end(); ++it)
+	    {
+		it->oldPName.assign( it->oldPName.size(), ' ' );
+		it->newPName.assign( it->newPName.size(), ' ' );
+	    }
+	}
+    }
+
+    operator bool () const
+    {
+	return renameList;
+    }
+
+    bool apply()
+    {
+	try
+	{
+	    while(last != renameList->end())
+	    {
+		// backing store rename.
+		rDebug("renaming %s -> %s",
+			last->oldCName.c_str(), last->newCName.c_str());
+
+		// internal node rename..
+		dn->renameNode( last->oldPName.c_str(), 
+			        last->newPName.c_str() );
+
+		// rename on disk..
+		if(::rename( last->oldCName.c_str(), 
+			     last->newCName.c_str() ) == -1)
+		{
+		    rWarning("Error renaming %s: %s",
+			    last->oldCName.c_str(), strerror( errno ));
+		    dn->renameNode( last->newPName.c_str(), 
+			    last->oldPName.c_str(), false );
+		    return false;
+		}
+
+		++last;
+	    }
+
+	    return true;
+	} catch( rlog::Error &err )
+	{
+	    err.log( _RLWarningChannel );
+	    return false;
+	}
+    }
+
+    void undo()
+    {
+	rDebug("in undoRename");
+
+	if(last == renameList->begin())
+	{
+	    rDebug("nothing to undo");
+	    return; // nothing to undo
+	}
+
+	// list has to be processed backwards, otherwise we may rename
+	// directories and directory contents in the wrong order!
+	int undoCount = 0;
+	list<RenameEl>::const_iterator it = last;
+
+	while( it != renameList->begin() )
+	{
+	    --it;
+
+	    rDebug("undo: renaming %s -> %s", 
+		    it->newCName.c_str(), it->oldCName.c_str());
+
+	    ::rename( it->newCName.c_str(), it->oldCName.c_str() );
+	    try
+	    {
+		dn->renameNode( it->newPName.c_str(), 
+			        it->oldPName.c_str(), false );
+	    } catch( rlog::Error &err )
+	    {
+		err.log( _RLWarningChannel );
+		// continue on anyway...
+	    }
+	    ++undoCount;
+	};
+    
+	rWarning("Undo rename count: %i", undoCount);
+    }
+};
+
+DirNode::DirNode(EncFS_Context *_ctx,
+	const string &sourceDir, const shared_ptr<Config> &_config)
 {
     pthread_mutex_init( &mutex, 0 );
     
     Lock _lock( mutex );
 
+    ctx = _ctx;
     rootDir = sourceDir;
     config = _config;
 
@@ -268,41 +299,11 @@ DirNode::DirNode(const string &sourceDir, const Ptr<Config> &_config)
     if( rootDir[ rootDir.length()-1 ] != '/' )
 	rootDir.append( 1, '/');
 
-    naming = config->nameCoding.get();
-    inactivityTimer = config->inactivityTimer;
-
-    lastAccess = inactivityTimer ? time(0) : 0;
-
-    recentNodes.resize( RecentNodeCacheSize, (FileNode*)0 );
-    nextCacheLoc = 0;
+    naming = config->nameCoding;
 }
 
 DirNode::~DirNode()
 {
-#if CACHE_STATS
-    cerr << "cache stats: " << cacheMiss << " misses, hits by age:\n";
-#endif
-    for(int i=0; i<RecentNodeCacheSize; ++i)
-    {
-#if CACHE_STATS
-	cerr << "age " << i << " : " << cacheHits[i] << " hits\n";
-#endif
-	if(recentNodes[i] != NULL)
-	{
-	    FileNodeDestructor( recentNodes[i] );
-	    recentNodes[i] = NULL;
-	}
-    }
-
-    if(!openFiles.empty())
-    {
-	rError("Leaked FileNodes: %i", (int)openFiles.size());
-	CacheMap::iterator it = openFiles.begin();
-	for(; it!= openFiles.end(); ++it)
-	{
-	    rError("Leaked node: %s", it->second->cipherName());
-	}
-    }
 }
 
 bool
@@ -323,16 +324,12 @@ DirNode::rootDirectory()
 string 
 DirNode::cipherPath( const char *plaintextPath )
 {
-    ATIME_ATOMIC_UPDATE
-
     return rootDir + naming->encodePath( plaintextPath );
 }
 
 string
 DirNode::plainPath( const char *cipherPath_ )
 {
-    ATIME_ATOMIC_UPDATE
-
     try
     {
 	if( !strncmp( cipherPath_, rootDir.c_str(), 
@@ -364,8 +361,6 @@ DirNode::plainPath( const char *cipherPath_ )
 string
 DirNode::relativeCipherPath( const char *plaintextPath )
 {
-    ATIME_ATOMIC_UPDATE
-
     try
     {
 	if(plaintextPath[0] == '/')
@@ -388,16 +383,14 @@ DirNode::relativeCipherPath( const char *plaintextPath )
 
 DirTraverse DirNode::openDir(const char *plaintextPath)
 {
-    ATIME_ATOMIC_UPDATE
-
     string cyName = rootDir + naming->encodePath( plaintextPath );
     //rDebug("openDir on %s", cyName.c_str() );
 
-    DIR *dp = ::opendir( cyName.c_str() );
+    shared_ptr<DIR> dp( ::opendir( cyName.c_str() ), DirDeleter() );
     if(!dp)
     {
 	rDebug("opendir error %s", strerror(errno));
-	return DirTraverse( 0, 0, 0 );
+	return DirTraverse( dp, 0, shared_ptr<NameIO>() );
     } else
     {
 	uint64_t iv = 0;
@@ -416,8 +409,8 @@ DirTraverse DirNode::openDir(const char *plaintextPath)
     }
 }
 
-bool DirNode::genRenameList( list<RenameEl> &renameList, const char *fromP,
-	const char *toP )
+bool DirNode::genRenameList( list<RenameEl> &renameList, 
+	const char *fromP, const char *toP )
 {
     uint64_t fromIV = 0, toIV = 0;
 
@@ -434,12 +427,13 @@ bool DirNode::genRenameList( list<RenameEl> &renameList, const char *fromP,
 
     // generate the real destination path, where we expect to find the files..
     rDebug("opendir %s", sourcePath.c_str() );
-    DIR *dir = opendir( sourcePath.c_str() );
-    if(dir == NULL)
+    shared_ptr<DIR> dir = shared_ptr<DIR>(
+	    opendir( sourcePath.c_str() ), DirDeleter() );
+    if(!dir)
 	return false;
     
     struct dirent *de = NULL;
-    while((de = ::readdir( dir )) != NULL)
+    while((de = ::readdir( dir.get() )) != NULL)
     {
 	// decode the name using the oldIV
 	uint64_t localIV = fromIV;
@@ -500,7 +494,6 @@ bool DirNode::genRenameList( list<RenameEl> &renameList, const char *fromP,
 			    ren.oldPName.c_str(), 
 			    ren.newPName.c_str()))
 		{
-		    closedir( dir );
 		    return false;
 		}
 	    }
@@ -520,122 +513,37 @@ bool DirNode::genRenameList( list<RenameEl> &renameList, const char *fromP,
 
 	    // abort.. Err on the side of safety and disallow rename, rather
 	    // then loosing files..
-	    closedir( dir );
 	    return false;
 	}
     }
-    closedir( dir );
+
     return true;
 }
 
-void DirNode::undoRename( list<RenameEl> &renameList,
-	list<RenameEl>::const_iterator &undoEnd)
-{
-    rDebug("in undoRename");
-
-    if(undoEnd == renameList.begin())
-    {
-	rDebug("nothing to undo");
-	return; // nothing to undo
-    }
-
-    // list has to be processed backwards, otherwise we may rename directories
-    // and directory contents in the wrong order!
-    int undoCount = 0;
-    list<RenameEl>::const_iterator it = undoEnd;
-
-    while( it != renameList.begin() )
-    {
-	--it;
-
-	rDebug("undo: renaming %s -> %s", 
-		it->newCName.c_str(), it->oldCName.c_str());
-
-	::rename( it->newCName.c_str(), it->oldCName.c_str() );
-	try
-	{
-	    renameNode( it->newPName.c_str(), it->oldPName.c_str(), false );
-	} catch( rlog::Error &err )
-	{
-	    err.log( _RLWarningChannel );
-	    // continue on anyway...
-	}
-	++undoCount;
-    };
-    
-    rWarning("Undo rename count: %i", undoCount);
-}
-
-void DirNode::cleanList( list<RenameEl> &renameList )
-{
-    // now we've potentially got a bunch of decoded filenames sitting in
-    // memory..  do a little cleanup before leaving..
-    list<RenameEl>::iterator it;
-    for(it = renameList.begin(); it != renameList.end(); ++it)
-    {
-	it->oldPName.assign( it->oldPName.size(), ' ' );
-	it->newPName.assign( it->newPName.size(), ' ' );
-    }
-}
 
 /*
     A bit of a pain.. If a directory is renamed in a filesystem with
     directory initialization vector chaining, then we have to recursively
     rename every descendent of this directory, as all initialization vectors
     will have changed..
+
+    Returns a list of renamed items on success, a null list on failure.
 */
-bool DirNode::recursiveRename( const char *fromP, const char *toP )
+shared_ptr<RenameOp>
+DirNode::newRenameOp( const char *fromP, const char *toP )
 {
     // Do the rename in two stages to avoid chasing our tail
     // Undo everything if we encounter an error!
-    list<RenameEl> renameList;
-    bool ok = true;
-    if(!genRenameList( renameList, fromP, toP ))
+    shared_ptr< list<RenameEl> > renameList(new list<RenameEl>);
+    if(!genRenameList( *renameList.get(), fromP, toP ))
     {
 	rWarning("Error during generation of recursive rename list");
-	cleanList( renameList );
-	ok = false;
+	return shared_ptr<RenameOp>();
     } else
-    {
-	list<RenameEl>::const_iterator it = renameList.begin();
-
-	try
-	{
-	    // start renaming, keeping track of an undo point in case we have to
-	    // stop
-	    for(it = renameList.begin(); it != renameList.end(); ++it)
-	    {
-		// backing store rename.
-		rDebug("renaming %s -> %s",
-			it->oldCName.c_str(), it->newCName.c_str());
-
-		// internal node rename..
-		renameNode( it->oldPName.c_str(), it->newPName.c_str() );
-
-		// rename on disk..
-		if(::rename( it->oldCName.c_str(), it->newCName.c_str() ) == -1)
-		{
-		    rWarning("Error renaming %s: %s",
-			    it->oldCName.c_str(), strerror( errno ));
-		    renameNode( it->newPName.c_str(), 
-			    it->oldPName.c_str(), false );
-		    ok = false;
-		    break;
-		}
-	    }
-	} catch( rlog::Error &err )
-	{
-	    err.log( _RLWarningChannel );
-	    ok = false;
-	}
-
-	if(!ok)
-	    undoRename( renameList, it );
-	cleanList( renameList );
-    }
-    return ok;
+	return shared_ptr<RenameOp>( new RenameOp(this, renameList) );
 }
 
+
 int DirNode::mkdir(const char *plaintextPath, mode_t mode, 
 	uid_t uid, gid_t gid)
 {
@@ -683,27 +591,23 @@ DirNode::rename( const char *fromPlaintext, const char *toPlaintext )
     
     rLog( Info, "rename %s -> %s", fromCName.c_str(), toCName.c_str() );
    
-    bool inMap = false;
-    FileNode *toNode = findNode( toPlaintext, &inMap );
+    shared_ptr<FileNode> toNode = findOrCreate( toPlaintext );
 
-    if(toNode && inMap != false)
-    {
-	rError("Error, attempting to rename %s over existing open file %s !",
-		fromCName.c_str(), toCName.c_str() );
-	return -EACCES;
-    }
-
-    bool didRecursiveRename = false;
+    shared_ptr<RenameOp> renameOp;
     if( hasDirectoryNameDependency() && isDirectory( fromCName.c_str() ))
     {
 	rLog( Info, "recursive rename begin" );
-	if(! recursiveRename( fromPlaintext, toPlaintext ))
+	renameOp = newRenameOp( fromPlaintext, toPlaintext );
+
+	if(!renameOp || !renameOp->apply())
 	{
+	    if(renameOp)
+		renameOp->undo();
+
 	    rWarning("rename aborted");
 	    return -EACCES;
 	}
 	rLog( Info, "recursive rename end" );
-	didRecursiveRename = true;
     }
 
     int res = 0;
@@ -717,6 +621,9 @@ DirNode::rename( const char *fromPlaintext, const char *toPlaintext )
 	    // undo
 	    res = -errno;
 	    renameNode( toPlaintext, fromPlaintext, false );
+
+	    if(renameOp)
+		renameOp->undo();
 	}
     } catch( rlog::Error &err )
     {
@@ -762,177 +669,22 @@ int DirNode::link( const char *from, const char *to )
     return res;
 }
 
-bool 
-DirNode::hasOpenFiles()
-{
-    Lock _lock( mutex );
-    bool result = !openFiles.empty();
-
-    return result;
-}
-
-string
-DirNode::openFileList()
-{
-    Lock _lock( mutex );
-
-    string result;
-
-    CacheMap::const_iterator it = openFiles.begin();
-    CacheMap::const_iterator end = openFiles.end();
-    for(; it != end; ++it)
-    {
-	result += it->second->cipherName();
-	result += "\n";
-    }
-
-    return result;
-}
-
-int
-DirNode::idleSeconds()
-{
-    if(inactivityTimer)
-    {
-	Lock _lock( mutex );
-	time_t current = time(0);
-	return int( current - lastAccess );
-    } else
-	return 0;
-}
-
-void FileNodeDestructor( FileNode *fnode )
-{
-    if(fnode == NULL)
-	return;
-
-    Lock fnodeLock( fnode->mutex );
-
-    rLog( Info, "in FileNodeDestructor for %s (refcount %i)",
-	    fnode->cipherName(), fnode->refCnt);
-
-    if( fnode->refCnt <= 0 )
-	rError("Error, fnode %s refcount = %i before release",
-		fnode->cipherName(), fnode->refCnt );
-
-    // don't destroy nodes which are in the cache.  Only release() removes from
-    // the cache, and then the node can be destroyed..
-    if( (--fnode->refCnt == 0) && (fnode->retainCount == 0) )
-    {
-	rLog( Info, "destroying FNode %s", fnode->cipherName());
-
-	// mutex held going into destructor..
-	fnodeLock.leave();
-	delete fnode;
-    }
-}
-
 /*
-    Release should be called the same number of times as openNode()
+    The node is keyed by filename, so a rename means the internal node names
+    must be changed.
 */
-void
-DirNode::release( const char *plaintextPath )
-{
-#if REQUEST_HISTORY
-    addHistory( plaintextPath, "release" );
-#endif
-
-    rLog( Info, "releasing %s", naming->encodePath( plaintextPath ).c_str() );
-    
-    Lock _lock( mutex );
-
-    CacheMap::iterator it = openFiles.find( HASHKEY(plaintextPath) );
-
-    if(it != openFiles.end())
-    {
-	FileNode *fnode = it->second;
-	rAssert(fnode != NULL);
-
-	// some trickly locking stuff going on here...
-	Lock fnodeLock( fnode->mutex );
-
-	if( --fnode->retainCount == 0 )
-	{
-	    rLog( Info, "removing FileNode %s from map", 
-		    naming->encodePath( plaintextPath ).c_str());
-	
-	    // storedName tries to make use of shallow copy to clear memory
-	    // used for storing filenames.. Not really sure if it does any
-	    // good.
-	    string storedName = it->first;
-	    openFiles.erase( it );
-	    storedName.assign( storedName.length(), '\0' );
-
-	    if(fnode->refCnt == 0)
-	    {
-		// mutex held going into ~FileNode.  we never unlock it..
-		fnodeLock.leave();
-		delete fnode;
-	    }
-	}
-    } else
-    {
-	rWarning("unexpected release call for %s",
-		naming->encodePath( plaintextPath ).c_str());
-#if REQUEST_HISTORY
-	showHistory( plaintextPath );
-#endif
-    }
-}
-
-static void removeFromCache( std::vector<FileNode*> &cache,
-	FileNode *node )
-{
-    for(int i=0; i<RecentNodeCacheSize; ++i)
-    {
-	if( cache[i] == node )
-	{
-	    cache[i] = NULL;
-	    FileNodeDestructor( node );
-	    break;
-	}
-    }
-}
-
-static void removeFromCache( std::vector<FileNode*> &cache,
-	const char *plainName )
-{
-    for(int i=0; i<RecentNodeCacheSize; ++i)
-    {
-	if( cache[i] != NULL && !strcmp(cache[i]->plaintextName(), plainName ))
-	{
-	    FileNode *fnode = cache[i];
-	    cache[i] = NULL;
-	    FileNodeDestructor( fnode );
-	    break;
-	}
-    }
-}
-
-/*
-    rename just internal cache nodes.  The node is keyed by filename, so a
-    rename means the internal node names must be changed.
-
-*/
-FileNode * DirNode::renameNode( const char *from, const char *to )
+shared_ptr<FileNode> DirNode::renameNode( const char *from, const char *to )
 {
     return renameNode( from, to, true );
 }
 
-FileNode * DirNode::renameNode( const char *from, const char *to, 
+shared_ptr<FileNode> DirNode::renameNode( const char *from, const char *to, 
 	bool forwardMode )
 {
-    // don't want there to be two nodes with the same name!
-    removeFromCache( recentNodes, to );
-
-    bool inMap = false;
-    bool created = false;
-    FileNode *node = findOrCreate( from, "renameNode", &inMap, &created );
+    shared_ptr<FileNode> node = findOrCreate( from );
 
     if(node)
     {
-	Lock nodeLock( node->mutex );
-
 	uint64_t newIV = 0;
 	string cname = rootDir + naming->encodePath( to, &newIV );
 
@@ -941,97 +693,30 @@ FileNode * DirNode::renameNode( const char *from, const char *to,
 
 	if(node->setName( to, cname.c_str(), newIV, forwardMode ))
 	{
-	    if(inMap)
-	    {
-		openFiles.erase( HASHKEY(from) );
-		openFiles.insert( make_pair( HASHKEY(to), node ) );
-	    }
+	    if(ctx)
+		ctx->renameNode( from, to );
 	} else
 	{
 	    // rename error! - put it back 
 	    rError("renameNode failed");
-	    // TODO: leaks a node reference count here...
 	    throw ERROR("Internal node name change failed!");
 	}
     }
 
-    FileNodeDestructor( node );
-
     return node;
 }
 
-FileNode *DirNode::findNode( const char *plainName, bool *inMap )
+shared_ptr<FileNode> DirNode::findOrCreate( const char *plainName)
 {
-    rAssert( inMap != NULL );
-
-    FileNode *node = NULL;
-
-    // first check the cache node list, to avoid unecessary thrashing..
-    for(int i=0; i<RecentNodeCacheSize; ++i)
-    {
-	FileNode *tmpNode = recentNodes[i];
-	if(tmpNode != NULL && !strcmp(tmpNode->plaintextName(), plainName))
-	{
-	    node = tmpNode;
-#if CACHE_STATS
-	    int cacheNum = (nextCacheLoc + RecentNodeCacheSize-1 - i) %
-		RecentNodeCacheSize;
-	    ++cacheHits[ cacheNum ];
-#endif
-	    *inMap = false;
-	    break;
-	}
-    }
+    shared_ptr<FileNode> node;
+    if(ctx)
+	node = ctx->lookupNode( plainName );
 
     if(!node)
-    {
-#if CACHE_STATS
-	++cacheMiss;
-#endif
-
-	// check if the name is in the open-file cache..
-	CacheMap::const_iterator it = openFiles.find( HASHKEY(plainName) );
-
-	if(it != openFiles.end())
-	{
-	    node = it->second;
-	    rAssert(node != NULL);
-	    *inMap = true;
-	}
-    }
-
-    return node;
-}
-
-FileNode *DirNode::findOrCreate( const char *plainName, const char *requestor,
-	bool *inMap, bool *created )
-{
-    rAssert( inMap != NULL );
-    rAssert( created != NULL );
-
-#if REQUEST_HISTORY
-    addHistory( plainName, requestor );
-#else
-    // don't complain about unused vars..
-    (void)requestor;
-#endif
-
-    // don't use ATIME_ATOMIC_UPDATE macro because we're already in locked
-    // region
-    if(inactivityTimer)
-	lastAccess = time(0);
-    
-    FileNode *node = findNode( plainName, inMap );
-
-    if(node)
-    {	
-	node->incRef();
-	*created = false;
-    } else
     {
 	uint64_t iv = 0;
 	string cipherName = naming->encodePath( plainName, &iv );
-	node = new FileNode( this, 
+	node.reset( new FileNode( this, 
 		config->fsSubVersion,
 		plainName, 
 		(rootDir + cipherName).c_str(), 
@@ -1040,46 +725,28 @@ FileNode *DirNode::findOrCreate( const char *plainName, const char *requestor,
 		config->blockMACRandBytes, 
 		config->uniqueIV,
 		config->externalIVChaining,
-		config->forceDecode);
+		config->forceDecode,
+		config->reverseEncryption,
+                config->allowHoles) );
 		
 	if(config->externalIVChaining)
 	    node->setName(0, 0, iv);
 
 	rLog(Info, "created FileNode for %s", node->cipherName());
-	*inMap = false;
-	*created = true;
     }
 
     return node;
 }
 
-Ptr<FileNode>
+shared_ptr<FileNode>
 DirNode::lookupNode( const char *plainName, const char * requestor )
 {
+    (void)requestor;
     Lock _lock( mutex );
 
-    bool inMap = false;
-    bool created = false;
-    FileNode *node = findOrCreate( plainName, requestor, &inMap, &created );
+    shared_ptr<FileNode> node = findOrCreate( plainName );
 
-    if(created && node && !inMap)
-    {
-	// retain a copy in in recent nodes list..
-	if( recentNodes[ nextCacheLoc ] )
-	    FileNodeDestructor( recentNodes[ nextCacheLoc ] );
-
-	node->incRef();
-
-	recentNodes[ nextCacheLoc ] = node;
-	if(++nextCacheLoc >= RecentNodeCacheSize)
-	    nextCacheLoc = 0;
-    }
-
-    // Return a fresh Ptr<> wrapper.  We don't share wrappers because that
-    // would introduce race conditions on the wrapper.  We use it only to
-    // ensure that an appropriate file node destructor is called when the
-    // caller finishes with it.
-    return Ptr<FileNode>(node, FileNodeDestructor);
+    return node;
 }
 
 /*
@@ -1087,78 +754,47 @@ DirNode::lookupNode( const char *plainName, const char * requestor )
     node on sucess..  This is done in one step to avoid any race conditions
     with the stored state of the file.
 */
-Ptr<FileNode>
+shared_ptr<FileNode>
 DirNode::openNode( const char *plainName, const char * requestor, int flags,
 	int *result )
 {
+    (void)requestor;
     rAssert( result != NULL );
     Lock _lock( mutex );
 
-    bool inMap = false;
-    bool created = false;
-    FileNode *node = findOrCreate( plainName, requestor, &inMap, &created );
+    shared_ptr<FileNode> node = findOrCreate( plainName );
 
-    if( (*result = node->open( flags )) >= 0 )
-    {
-	node->incRetain();
-    
-	if(!inMap)
-	{
-	    openFiles.insert( make_pair( HASHKEY(plainName), node ) );
-	    if(!created)
-		removeFromCache( recentNodes, node );
-	}
-
-       	return Ptr<FileNode>(node, FileNodeDestructor);
-    } else
-    {
-	// error.. undo..
-	FileNodeDestructor( node );
-
-	return Ptr<FileNode>();
-    }
+    if( node && (*result = node->open( flags )) >= 0 )
+	return node;
+    else
+	return shared_ptr<FileNode>();
 }
 
 int DirNode::unlink( const char *plaintextName )
 {
-    rLog( Info, "unlink %s", naming->encodePath( plaintextName ).c_str() );
+    string cyName = naming->encodePath( plaintextName );
+    rLog( Info, "unlink %s", cyName.c_str() );
 
     Lock _lock( mutex );
-    
+   
     int res = 0;
-
-    CacheMap::const_iterator it = openFiles.find( HASHKEY(plaintextName) );
-    if(it != openFiles.end())
+    if(ctx && ctx->lookupNode( plaintextName ))
     {
-	// If we aren't using a version of FUSE with file-hide support, then we
-	// can't allow the unlink of an open file!
-	/*
-	    This happens if a file is retained in the cache. 
-
-	    The only reason for a file to be retained now is because it was
-	    opened and we're keeping it around until it if fully released.
-	    
-	    We don't have a way to support Unix's delete while open semantics
-	    because then we'd be in trouble if another open came along. 
-	*/
-	rWarning("Refusing to unlink cached file: %s (%i ref, %i retain)",
-		it->second->cipherName(),
-		it->second->refCnt, it->second->retainCount);
+	// If FUSE is running with "hard_remove" option where it doesn't
+	// hide open files for us, then we can't allow an unlink of an open
+	// file..
+	rWarning("Refusing to unlink open file: %s, hard_remove option "
+		"is probably in effect", cyName.c_str() );
 	res = -EBUSY;
-    }
-
-    if(res == 0)
+    } else
     {
-	removeFromCache( recentNodes, plaintextName );
-
-	string cyName = rootDir + naming->encodePath( plaintextName );
-	res = ::unlink( cyName.c_str() );
+	string fullName = rootDir + cyName;
+	res = ::unlink( fullName.c_str() );
 	if(res == -1)
 	{
 	    res = -errno;
 	    rDebug("unlink error: %s", strerror(errno));
-	} else
-	    res = 0;
+	} 
     }
 	
     return res;
