@@ -28,22 +28,21 @@
 #include <string>
 
 #include "FileNode.h"
-#include "LinkedOwner.h"
 #include "NameIO.h"
+#include "CipherKey.h"
 
-#ifdef USE_HASHMAP
-#include <ext/hash_map>
-#endif
-
-using rel::OpaqueValue;
+using boost::shared_ptr;
 
 class Cipher;
-typedef OpaqueValue CipherKey;
+class RenameOp;
+struct RenameEl;
+class EncFS_Context;
 
-class DirTraverse : public rel::LinkedOwner
+class DirTraverse
 {
 public:
-    DirTraverse(DIR *dirPtr, uint64_t iv, const NameIO *naming);
+    DirTraverse(const shared_ptr<DIR> &dirPtr, uint64_t iv, 
+	        const shared_ptr<NameIO> &naming);
     DirTraverse(const DirTraverse &src);
     ~DirTraverse();
 
@@ -63,11 +62,12 @@ public:
     */
     std::string nextInvalid();
 private:
-    DIR *dir; // struct DIR
+
+    shared_ptr<DIR> dir; // struct DIR
     // initialization vector to use.  Not very general purpose, but makes it
     // more efficient to support filename IV chaining..
     uint64_t iv; 
-    const NameIO *naming;
+    shared_ptr<NameIO> naming;
 };
 inline bool DirTraverse::valid() const { return dir != 0; }
 
@@ -89,9 +89,9 @@ class DirNode
 public:
     struct Config
     {
-	Ptr<Cipher> cipher; // cipher to use
+	shared_ptr<Cipher> cipher; // cipher to use
 	CipherKey key; // cipher key to use
-	Ptr<NameIO> nameCoding; // filename encoding implementation
+	shared_ptr<NameIO> nameCoding; // filename encoding implementation
 	int fsSubVersion; // filesystem version number at creation
 	int blockSize; // file data block size
 	bool inactivityTimer; // enables inactivity tracking
@@ -100,7 +100,8 @@ public:
 	bool uniqueIV; // enable per-file initialization vectors
 	bool externalIVChaining;
 	bool forceDecode; // force decoding, even if errors are detected
-
+	bool reverseEncryption;  
+        bool allowHoles; // allow holes in files
 	Config()
 	    : fsSubVersion(0)
 	    , blockSize(1)
@@ -110,27 +111,29 @@ public:
 	    , uniqueIV( false )
 	    , externalIVChaining( false )
 	    , forceDecode( false )
+	    , reverseEncryption ( false )
+            , allowHoles( false )
 	    { }
     };
 
     // sourceDir points to where raw files are stored
-    DirNode( const std::string &sourceDir,  const Ptr<Config> &config );
+    DirNode( EncFS_Context *ctx,
+	    const std::string &sourceDir,  const shared_ptr<Config> &config );
     ~DirNode();
 
     // return the path to the root directory
     std::string rootDirectory();
 
     // find files
-    Ptr<FileNode> lookupNode( const char *plaintextName, 
+    shared_ptr<FileNode> lookupNode( const char *plaintextName, 
 	                      const char *requestor );
 
     /*
 	Combined lookupNode + node->open() call.  If the open fails, then the
-	node is not retained.  If the open succeeds, then the node is returned
-	and the node is retained in the cache..
+	node is not retained.  If the open succeeds, then the node is returned.
     */
-    Ptr<FileNode> openNode( const char *plaintextName, const char *requestor,
-	    int flags, int *openResult );
+    shared_ptr<FileNode> openNode( const char *plaintextName, 
+	    const char *requestor, int flags, int *openResult );
 
     std::string cipherPath( const char *plaintextPath );
     std::string plainPath( const char *cipherPath );
@@ -147,9 +150,6 @@ public:
     */
     bool hasDirectoryNameDependency() const;
 
-    // release file
-    void release( const char *plaintextName );
-
     // unlink the specified file
     int unlink( const char *plaintextName );
 
@@ -164,25 +164,20 @@ public:
 
     int link( const char *from, const char *to );
     
-    // returns true if there are unreleased files
-    bool hasOpenFiles();
-  
-    // returns \n separated list of open filenames.. for debugging..
-    std::string openFileList();
-
     // returns idle time of filesystem in seconds
     int idleSeconds();
 
-
 protected:
 
     /*
 	notify that a file is being renamed. 
 	This renames the internal node, if any.  If the file is not open, then
 	this call has no effect.
-	Returns the FileNode if it was found in the cache map.
+	Returns the FileNode if it was found.
     */
-    FileNode * renameNode( const char *from, const char *to );
+    shared_ptr<FileNode> renameNode( const char *from, const char *to );
+    shared_ptr<FileNode> renameNode( const char *from, const char *to, 
+	                             bool forwardMode );
 
     /*
 	when directory IV chaining is enabled, a directory can't be renamed
@@ -190,58 +185,27 @@ protected:
 	called after renaming the directory, passing in the plaintext from and
 	to paths.
     */
-    bool recursiveRename( const char *from, const char *to );
+    shared_ptr<RenameOp> newRenameOp( const char *from, const char *to );
 
 private:
 
-    struct RenameEl
-    {
-	// ciphertext names
-	std::string oldCName;
-	std::string newCName; // intermediate name (not the final cname)
-
-	// plaintext names
-	std::string oldPName;
-	std::string newPName;
-
-	bool isDirectory;
-    };
-    
-    FileNode * renameNode( const char *from, const char *to, bool forwardMode );
+    friend class RenameOp;
 
     bool genRenameList( std::list<RenameEl> &list, const char *fromP,
 	    const char *toP );
-    void undoRename( std::list<RenameEl> &, 
-	    std::list<RenameEl>::const_iterator &undoEnd);
-    FileNode *findNode( const char *plainName, bool *inMap );
-    FileNode *findOrCreate( const char *plainName, const char *requestor,
-	    bool *inMap, bool *created );
-
-    static void cleanList( std::list<RenameEl> &list );
-
-    // This is a cache for a few recent unopened nodes.
-    std::vector<FileNode*> recentNodes;
-    int nextCacheLoc;
+    
+    shared_ptr<FileNode> findOrCreate( const char *plainName);
 
     pthread_mutex_t mutex;
 
+    EncFS_Context *ctx;
+
     // passed in as configuration
     std::string rootDir;
-    Ptr<Config> config;
-
-    // Cache for open nodes, which stay arround indefinately until a release is
-    // called.
-#ifdef USE_HASHMAP
-    typedef __gnu_cxx::hash_map<std::string, FileNode*> CacheMap;
-#else
-    typedef std::map<std::string, FileNode*> CacheMap;
-#endif
-    CacheMap openFiles;
-    time_t lastAccess;
+    shared_ptr<Config> config;
 
     // stored here to reduce access through config var..
-    NameIO *naming;
-    bool inactivityTimer;
+    shared_ptr<NameIO> naming;
 };
 
 #endif
