@@ -28,12 +28,14 @@
 #include "autosprintf.h"
 
 #include "FileUtils.h"
-#include "Config.h"
+#include "ConfigReader.h"
 
 #include "DirNode.h"
 #include "Cipher.h"
 #include "StreamNameIO.h"
 #include "BlockNameIO.h"
+#include "NullNameIO.h"
+#include "Context.h"
 
 #include <rlog/rlog.h>
 #include <rlog/Error.h>
@@ -48,18 +50,29 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <string.h>
 
 #include <iostream>
 #include <sstream>
 
 #include "i18n.h"
 
+#include <boost/filesystem/fstream.hpp>
+#include <boost/archive/xml_iarchive.hpp>
+#include <boost/archive/xml_oarchive.hpp>
+#include <boost/serialization/split_free.hpp>
+
 using namespace rel;
 using namespace rlog;
 using namespace std;
 using namespace gnu;
+namespace fs = boost::filesystem;
+
+static const int DefaultBlockSize = 1024;
+// The maximum length of text passwords.  If longer are needed,
+// use the extpass option, as extpass can return arbitrary length binary data.
+static const int MaxPassBuf = 512;
 
-static const int DefaultBlockSize = 512;
 // environment variable names for values encfs stores in the environment when
 // calling an external password program.
 static const char ENCFS_ENV_ROOTDIR[] = "encfs_root";
@@ -72,6 +85,8 @@ static const char ENCFS_ENV_STDERR[] = "encfs_stderr";
 static int V5SubVersion = 20040813; // fix MACFileIO block size issues
 static int V5SubVersionDefault = 0;
 
+const int V6SubVersion = 20080813; // switch to v6/XML, add allowHoles option
+
 struct ConfigInfo
 {
     const char *fileName;
@@ -83,17 +98,92 @@ struct ConfigInfo
     int currentSubVersion;
     int defaultSubVersion;
 } ConfigFileMapping[] = {
+    {".encfs6.xml", Config_V6, "ENCFS6_CONFIG", readV6Config, writeV6Config, 
+	V6SubVersion, 0 },
+    // backward compatible support for older versions
     {".encfs5", Config_V5, "ENCFS5_CONFIG", readV5Config, writeV5Config, 
 	V5SubVersion, V5SubVersionDefault },
-    // backward compatible support for config versions 3 and 4
     {".encfs4", Config_V4, NULL, readV4Config, writeV4Config, 0, 0 },
-    {".encfs3", Config_V3, NULL, readV3Config, writeV3Config, 0, 0 },
-    // forget about version 2 and 1 - from before the first public release!
+    // no longer support earlier versions
+    {".encfs3", Config_V3, NULL, NULL, NULL, 0, 0 },
     {".encfs2", Config_Prehistoric, NULL, NULL, NULL, 0, 0 },
     {".encfs", Config_Prehistoric, NULL, NULL, NULL, 0, 0 },
     {NULL,Config_None, NULL, NULL, NULL, 0, 0}};
 
 
+// define serialization helpers
+namespace boost
+{
+    namespace serialization
+    {
+        template<class Archive>
+        void save(Archive &ar, const EncFSConfig &cfg, 
+                unsigned int version)
+        {
+            (void)version;
+            ar << make_nvp("creator", cfg.creator);
+            ar << make_nvp("cipherAlg", cfg.cipherIface);
+            ar << make_nvp("nameAlg", cfg.nameIface);
+            ar << make_nvp("keySize", cfg.keySize);
+            ar << make_nvp("blockSize", cfg.blockSize);
+            ar << make_nvp("uniqueIV", cfg.uniqueIV);
+            ar << make_nvp("chainedNameIV", cfg.chainedNameIV);
+            ar << make_nvp("externalIVChaining", cfg.externalIVChaining);
+            ar << make_nvp("blockMACBytes", cfg.blockMACBytes);
+            ar << make_nvp("blockMACRandBytes", cfg.blockMACRandBytes);
+            ar << make_nvp("allowHoles", cfg.allowHoles);
+
+            int keyLen = cfg.keyData.length();
+            ar << make_nvp("encodedKeySize", keyLen);
+            char key[keyLen];
+            memcpy(key, cfg.keyData.data(), keyLen);
+            ar << make_nvp("encodedKeyData", make_binary_object(key, keyLen));
+        }
+
+        template<class Archive>
+        void load(Archive &ar, EncFSConfig &cfg, unsigned int version)
+        {
+            (void)version;
+            cfg.subVersion = version;
+            ar >> make_nvp("creator", cfg.creator);
+            ar >> make_nvp("cipherAlg", cfg.cipherIface);
+            ar >> make_nvp("nameAlg", cfg.nameIface);
+            ar >> make_nvp("keySize", cfg.keySize);
+            ar >> make_nvp("blockSize", cfg.blockSize);
+            ar >> make_nvp("uniqueIV", cfg.uniqueIV);
+            ar >> make_nvp("chainedNameIV", cfg.chainedNameIV);
+            ar >> make_nvp("externalIVChaining", cfg.externalIVChaining);
+            ar >> make_nvp("blockMACBytes", cfg.blockMACBytes);
+            ar >> make_nvp("blockMACRandBytes", cfg.blockMACRandBytes);
+            ar >> make_nvp("allowHoles", cfg.allowHoles);
+       
+            int encodedKeySize;
+            ar >> make_nvp("encodedKeySize", encodedKeySize);
+            char key[encodedKeySize];
+            ar >> make_nvp("encodedKeyData", 
+                    make_binary_object(key, encodedKeySize));
+            cfg.keyData.assign( (char*)key, encodedKeySize );
+        }
+        
+        template<class Archive>
+        void serialize(Archive &ar, EncFSConfig &cfg, unsigned int version)
+        {
+            split_free(ar, cfg, version);
+        }
+
+        template<class Archive>
+        void serialize(Archive &ar, Interface &i, const unsigned int version)
+        {
+            (void)version;
+            ar & make_nvp("name", i.name());
+            ar & make_nvp("major", i.current());
+            ar & make_nvp("minor", i.revision());
+        }
+    }
+}
+BOOST_CLASS_VERSION(EncFSConfig, V6SubVersion)
+
+
 EncFS_Root::EncFS_Root()
 {
 }
@@ -223,18 +313,44 @@ ConfigType readConfig( const string &rootDir, EncFSConfig *config )
     return Config_None;
 }
 
+bool readV6Config( const char *configFile, EncFSConfig *config,
+	ConfigInfo *info)
+{
+    (void)info;
+
+    fs::ifstream st(configFile);
+    if(st.is_open())
+    {
+        try
+        {
+            boost::archive::xml_iarchive ia( st );
+            ia >> BOOST_SERIALIZATION_NVP( *config );
+        
+            return true;
+        } catch(boost::archive::archive_exception &e)
+        {
+            rError("Archive exception: %s", e.what());
+            return false;
+        }
+    } else
+    {
+        rInfo("Failed to load config file %s", configFile);
+        return false;
+    }
+}
+
 bool readV5Config( const char *configFile, EncFSConfig *config,
 	ConfigInfo *info)
 {
     bool ok = false;
 
     // use Config to parse the file and query it..
-    Config cfgRdr;
+    ConfigReader cfgRdr;
     if(cfgRdr.load( configFile ))
     {
 	try
 	{
-	    config->subVersion = cfgRdr["subVersion"].readBER( 
+	    config->subVersion = cfgRdr["subVersion"].readInt( 
 		    info->defaultSubVersion );
 	    if(config->subVersion > info->currentSubVersion)
 	    {
@@ -245,6 +361,13 @@ bool readV5Config( const char *configFile, EncFSConfig *config,
 			config->subVersion, info->currentSubVersion);
 		return false;
 	    }
+	    if( config->subVersion < 20040813 )
+	    {
+		rError(_("This version of EncFS doesn't support "
+			    "filesystems created before 2004-08-13"));
+		return false;
+	    }
+
 	    cfgRdr["creator"] >> config->creator;
 	    cfgRdr["cipher"] >> config->cipherIface;
 	    cfgRdr["naming"] >> config->nameIface;
@@ -254,24 +377,9 @@ bool readV5Config( const char *configFile, EncFSConfig *config,
 	    config->uniqueIV = cfgRdr["uniqueIV"].readBool( false );
 	    config->chainedNameIV = cfgRdr["chainedIV"].readBool( false );
 	    config->externalIVChaining = cfgRdr["externalIV"].readBool( false );
-	    if( config->subVersion < 20040518 )
-	    {
-		bool blockMAC = cfgRdr["blockMAC"].readBool( false );
-		if(blockMAC)
-		{
-		    config->blockMACBytes = 8;
-		    config->blockMACRandBytes = 4;
-		} else
-		{
-		    config->blockMACBytes = 0;
-		    config->blockMACRandBytes = 0;
-		}
-	    } else
-	    {
-		config->blockMACBytes = cfgRdr["blockMACBytes"].readBER(0);
-		config->blockMACRandBytes = 
-		    cfgRdr["blockMACRandBytes"].readBER(0);
-	    }
+	    config->blockMACBytes = cfgRdr["blockMACBytes"].readInt(0);
+	    config->blockMACRandBytes = 
+		cfgRdr["blockMACRandBytes"].readInt(0);
 
 	    ok = true;
 	} catch( rlog::Error &err)
@@ -291,7 +399,7 @@ bool readV4Config( const char *configFile, EncFSConfig *config,
     bool ok = false;
 
     // use Config to parse the file and query it..
-    Config cfgRdr;
+    ConfigReader cfgRdr;
     if(cfgRdr.load( configFile ))
     {
 	try
@@ -323,44 +431,6 @@ bool readV4Config( const char *configFile, EncFSConfig *config,
     return ok;
 }
 
-bool readV3Config( const char *configFile, EncFSConfig *config,
-	ConfigInfo *info)
-{
-    (void)configFile;
-    // use Config to parse the file and query it..
-    // fill in default for V3
-    config->creator = "EncFS 0.x";
-    config->subVersion = info->defaultSubVersion;
-    config->cipherIface = Interface("ssl/blowfish-v0.2", 1, 0, 0);
-    config->nameIface = Interface("nameio/stream", 0, 1, 0);
-    config->keySize = 160;
-    config->blockSize = 64;
-    config->blockMACBytes = 0;
-    config->blockMACRandBytes = 0;
-    config->uniqueIV = false;
-    config->externalIVChaining = false;
-    config->chainedNameIV = false;
-
-    bool ok = false;
-    int vkeyFD = open( configFile, O_RDONLY );
-    if( vkeyFD >= 0 )
-    {
-	const int headerSize = 22; // 160 bit (20 bytes) + 2 byte checksum
-	char keyBuf [ headerSize ];
-
-	read( vkeyFD, keyBuf, headerSize );
-	close( vkeyFD );
-
-	config->keyData.assign( keyBuf, headerSize );
-	ok = true;
-    } else
-    {
-	rDebug("Error opening config file %s", configFile );
-    }
-
-    return ok;
-}
-
 bool saveConfig( ConfigType type, const string &rootDir,
 	EncFSConfig *config )
 {
@@ -396,9 +466,21 @@ bool saveConfig( ConfigType type, const string &rootDir,
     return ok;
 }
 
+bool writeV6Config( const char *configFile, EncFSConfig *config )
+{
+    fs::ofstream st( configFile );
+    if(!st.is_open())
+        return false;
+
+    boost::archive::xml_oarchive oa(st);
+    oa << BOOST_SERIALIZATION_NVP( config );
+
+    return true;
+}
+
 bool writeV5Config( const char *configFile, EncFSConfig *config )
 {
-    Config cfg;
+    ConfigReader cfg;
 
     cfg["creator"] << config->creator;
     cfg["subVersion"] << config->subVersion;
@@ -418,7 +500,7 @@ bool writeV5Config( const char *configFile, EncFSConfig *config )
 
 bool writeV4Config( const char *configFile, EncFSConfig *config )
 {
-    Config cfg;
+    ConfigReader cfg;
 
     cfg["cipher"] << config->cipherIface;
     cfg["keySize"] << config->keySize;
@@ -428,25 +510,6 @@ bool writeV4Config( const char *configFile, EncFSConfig *config )
     return cfg.save( configFile );
 }
 
-bool writeV3Config( const char *configFile, EncFSConfig *config )
-{
-    bool ok = true;
-
-    int fd = open( configFile, O_RDWR );
-    if(fd >= 0)
-    {
-	::pwrite(fd, config->keyData.data(), config->keyData.length(), 0);
-	close( fd );
-    }  else
-    {
-	rError(_("Error opening key file %s for write: %s"), configFile,
-		strerror( errno ));
-	ok = false;
-    }
-
-    return ok;
-}
-
 static
 Cipher::CipherAlgorithm findCipherAlgorithm(const char *name,
 	int keySize )
@@ -680,23 +743,35 @@ int selectBlockSize( const Cipher::CipherAlgorithm &alg )
     return blockSize;
 }
 
-static 
-void selectBlockMAC(int *macBytes, int *macRandBytes)
+static
+bool boolDefaultNo(const char *prompt)
 {
-    // xgroup(setup)
-    cout << _("Enable block authentication code headers\n"
-	"on every block in a file?  This adds about 12 bytes per block\n"
-	"to the storage requirements for a file, and significantly affects\n"
-	"performance but it also means [almost] any modifications or errors\n"
-	"within a block will be caught and will cause a read error.\n"
-	"The default here is No.  \n"
-	"Any response that does not begin with 'y' will mean No: ");
+    cout << prompt << "\n";
+    cout << _("The default here is No.\n"
+              "Any response that does not begin with 'y' will mean No: ");
 
     char answer[10];
     fgets( answer, sizeof(answer), stdin );
     cout << "\n";
 
-    if(tolower(answer[0]) == 'y')
+    if(tolower(answer[0]) == 'n')
+	return false;
+    else
+	return true;
+}
+
+static 
+void selectBlockMAC(int *macBytes, int *macRandBytes)
+{
+    // xgroup(setup)
+    bool addMAC = boolDefaultNo(
+            _("Enable block authentication code headers\n"
+	"on every block in a file?  This adds about 12 bytes per block\n"
+	"to the storage requirements for a file, and significantly affects\n"
+	"performance but it also means [almost] any modifications or errors\n"
+	"within a block will be caught and will cause a read error."));
+
+    if(addMAC)
     {
 	*macBytes = 8;
 
@@ -708,7 +783,8 @@ void selectBlockMAC(int *macBytes, int *macRandBytes)
 	    "vectors, which does not come with as great of performance\n"
 	    "penalty. \n"
 	    "Select a number of bytes, from 0 (no random bytes) to 8: ");
-    
+   
+        char answer[10];
 	int randSize = 0;
 	fgets( answer, sizeof(answer), stdin );
 	cout << "\n";
@@ -727,59 +803,12 @@ void selectBlockMAC(int *macBytes, int *macRandBytes)
     }
 }
 
-static 
-bool selectUniqueIV()
+static
+bool boolDefaultYes(const char *prompt)
 {
-    // xgroup(setup)
-    cout << _("Enable per-file initialization vectors?\n"
-	"This adds about 8 bytes per file to the storage requirements.\n"
-	"It should not affect performance except possibly with applications\n"
-	"which rely on block-aligned file io for performance.\n"
-	"The default here is Yes.  \n"
-	"Any response that does not begin with 'n' will mean Yes: ");
-
-    char answer[10];
-    fgets( answer, sizeof(answer), stdin );
-    cout << "\n";
-
-    if(tolower(answer[0]) == 'n')
-	return false;
-    else
-	return true;
-}
-
-static 
-bool selectChainedIV()
-{
-    // xgroup(setup)
-    cout << _("Enable filename initialization vector chaining?\n"
-	"This makes filename encoding dependent on the complete path, \n"
-	"rather then encoding each path element individually. \n"
-	"This is normally desireable, therefor the default is Yes. \n"
-	"Any response that does not begin with 'n' will mean Yes: ");
-
-    char answer[10];
-    fgets( answer, sizeof(answer), stdin );
-    cout << "\n";
-
-    if(tolower(answer[0]) == 'n')
-	return false;
-    else
-	return true;
-}
-
-static 
-bool selectExternalChainedIV()
-{
-    // xgroup(setup)
-    cout << _("Enable filename to IV header chaining?\n"
-	"This makes file data encoding dependent on the complete file path.\n"
-	"If a file is renamed, it will not decode sucessfully unless it\n"
-	"was renamed by encfs with the proper key.\n"
-	"If this option is enabled, then hard links will not be supported\n"
-	"in the filesystem.\n"
-	"The default is No. \n"
-	"Any response that does not begin with 'y' will mean No: ");
+    cout << prompt << "\n";
+    cout << _("The default here is Yes.\n"
+              "Any response that does not begin with 'n' will mean Yes: ");
 
     char answer[10];
     fgets( answer, sizeof(answer), stdin );
@@ -791,10 +820,53 @@ bool selectExternalChainedIV()
 	return false;
 }
 
-RootPtr createV5Config( const std::string &rootDir, 
+static 
+bool selectUniqueIV()
+{
+    // xgroup(setup)
+    return boolDefaultYes(
+            _("Enable per-file initialization vectors?\n"
+	"This adds about 8 bytes per file to the storage requirements.\n"
+	"It should not affect performance except possibly with applications\n"
+	"which rely on block-aligned file io for performance."));
+}
+
+static 
+bool selectChainedIV()
+{
+    // xgroup(setup)
+    return boolDefaultYes(
+            _("Enable filename initialization vector chaining?\n"
+	"This makes filename encoding dependent on the complete path, \n"
+	"rather then encoding each path element individually."));
+}
+
+static 
+bool selectExternalChainedIV()
+{
+    // xgroup(setup)
+    return boolDefaultNo(
+            _("Enable filename to IV header chaining?\n"
+	"This makes file data encoding dependent on the complete file path.\n"
+	"If a file is renamed, it will not decode sucessfully unless it\n"
+	"was renamed by encfs with the proper key.\n"
+	"If this option is enabled, then hard links will not be supported\n"
+	"in the filesystem."));
+}
+
+static 
+bool selectZeroBlockPassThrough()
+{
+    // xgroup(setup)
+    return boolDefaultNo(
+            _("Enable file-hole pass-through?\n"
+	"This avoids writing encrypted blocks when file holes are created."));
+}
+
+RootPtr createV6Config( EncFS_Context *ctx, const std::string &rootDir, 
 	bool enableIdleTracking, bool forceDecode,
 	const std::string &passwordProgram,
-	bool useStdin )
+	bool useStdin, bool reverseEncryption )
 {
     RootPtr rootInfo;
 
@@ -823,9 +895,25 @@ RootPtr createV5Config( const std::string &rootDir,
     bool uniqueIV = false;
     bool chainedIV = false;
     bool externalIV = false;
+    bool allowHoles = false;
+    
+    if (reverseEncryption)
+    {
+	uniqueIV = false;
+	chainedIV = false;
+	externalIV = false;
+	blockMACBytes = 0;
+	blockMACRandBytes = 0;
+    }
 
     if(answer[0] == 'p')
     {
+	if (reverseEncryption)
+	{
+	    rError(_("Paranoia configuration not supported for --reverse"));
+	    return rootInfo;
+	}
+
 	// xgroup(setup)
 	cout << _("Paranoia configuration selected.") << "\n";
 	// look for AES with 256 bit key..
@@ -847,18 +935,24 @@ RootPtr createV5Config( const std::string &rootDir,
     {
 	// xgroup(setup)
 	cout << _("Standard configuration selected.") << "\n";
-	// look for blowfish with 160 bit key..
-	// block name encoding is now standard as well..
-	// per-file initialization vectors are now standard, as they shouldn't
-	// have any significant performance penalty.
-	keySize = 160;
+	// AES w/ 192 bit key, block name encoding, per-file initialization
+        // vectors are all standard.
+	keySize = 192;
 	blockSize = DefaultBlockSize;
-	alg = findCipherAlgorithm("Blowfish", keySize);
-	nameIOIface = BlockNameIO::CurrentInterface();
+	alg = findCipherAlgorithm("AES", keySize);
 	blockMACBytes = 0;
-	uniqueIV = true;
-	chainedIV = true;
 	externalIV = false;
+	nameIOIface = BlockNameIO::CurrentInterface();
+
+	if (reverseEncryption)
+	{
+	    cout << _("--reverse specified, not using unique/chained IV") 
+		<< "\n";
+	} else
+	{
+	    uniqueIV = true;
+	    chainedIV = true;
+	}
     }
 
     if(answer[0] == 'x' || alg.name.empty())
@@ -881,23 +975,30 @@ RootPtr createV5Config( const std::string &rootDir,
 	keySize = selectKeySize( alg );
 	blockSize = selectBlockSize( alg );
 	nameIOIface = selectNameCoding();
-	chainedIV = selectChainedIV();
-	uniqueIV = selectUniqueIV();
-	if(chainedIV && uniqueIV)
-	    externalIV = selectExternalChainedIV();
-	else
+	if (reverseEncryption)
 	{
-	    // xgroup(setup)
-	    cout << _("External chained IV disabled, as both 'IV chaining'\n"
-		"and 'unique IV' features are required for this option.") 
-		<< endl;
-	    externalIV = false;
+	    cout << _("--reverse specified, not using unique/chained IV") << "\n";
+	} else
+	{
+	    chainedIV = selectChainedIV();
+	    uniqueIV = selectUniqueIV();
+	    if(chainedIV && uniqueIV)
+		externalIV = selectExternalChainedIV();
+	    else
+	    {
+		// xgroup(setup)
+		cout << _("External chained IV disabled, as both 'IV chaining'\n"
+		    "and 'unique IV' features are required for this option.") 
+                    << "\n";
+		externalIV = false;
+	    }
+	    selectBlockMAC(&blockMACBytes, &blockMACRandBytes);
+            allowHoles = selectZeroBlockPassThrough();
 	}
-	selectBlockMAC(&blockMACBytes, &blockMACRandBytes);
     }
 
-    Ptr<Cipher> cipher = Cipher::New( alg.name, keySize );
-    if(cipher.isNull())
+    shared_ptr<Cipher> cipher = Cipher::New( alg.name, keySize );
+    if(!cipher)
     {
 	rError(_("Unable to instanciate cipher %s, key size %i, block size %i"),
 		alg.name.c_str(), keySize, blockSize);
@@ -915,12 +1016,13 @@ RootPtr createV5Config( const std::string &rootDir,
     config.blockSize = blockSize;
     config.nameIface = nameIOIface;
     config.creator = "EncFS " VERSION;
-    config.subVersion = V5SubVersion;
+    config.subVersion = V6SubVersion;
     config.blockMACBytes = blockMACBytes;
     config.blockMACRandBytes = blockMACRandBytes;
     config.uniqueIV = uniqueIV;
     config.chainedNameIV = chainedIV;
     config.externalIVChaining = externalIV;
+    config.allowHoles = allowHoles;
 
     cout << "\n";
     // xgroup(setup)
@@ -970,20 +1072,20 @@ RootPtr createV5Config( const std::string &rootDir,
     config.keyData.assign( (char*)encodedKey, encodedKeySize );
     delete[] encodedKey;
 
-    if(volumeKey.isNull())
+    if(!volumeKey)
     {
 	rWarning(_("Failure generating new volume key! "
 		    "Please report this error."));
 	return rootInfo;
     }
 
-    if(!saveConfig( Config_V5, rootDir, &config ))
+    if(!saveConfig( Config_V6, rootDir, &config ))
 	return rootInfo;
 
     // fill in config struct
-    Ptr<NameIO> nameCoder = NameIO::New( config.nameIface, 
+    shared_ptr<NameIO> nameCoder = NameIO::New( config.nameIface, 
 	    cipher, volumeKey );
-    if(nameCoder.isNull())
+    if(!nameCoder)
     {
 	rWarning(_("Name coding interface not supported"));
 	cout << _("The filename encoding interface requested is not available") 
@@ -992,8 +1094,9 @@ RootPtr createV5Config( const std::string &rootDir,
     }
 	
     nameCoder->setChainedNameIV( config.chainedNameIV );
+    nameCoder->setReverseEncryption( reverseEncryption );
 
-    Ptr<DirNode::Config> dirNodeConfig (new DirNode::Config);
+    shared_ptr<DirNode::Config> dirNodeConfig (new DirNode::Config);
     dirNodeConfig->cipher = cipher;
     dirNodeConfig->key = volumeKey;
     dirNodeConfig->nameCoding = nameCoder;
@@ -1005,18 +1108,21 @@ RootPtr createV5Config( const std::string &rootDir,
     dirNodeConfig->uniqueIV = config.uniqueIV;
     dirNodeConfig->externalIVChaining = config.externalIVChaining;
     dirNodeConfig->forceDecode = forceDecode;
+    dirNodeConfig->reverseEncryption = reverseEncryption;
+    dirNodeConfig->allowHoles = config.allowHoles;
 
     rootInfo = RootPtr( new EncFS_Root );
     rootInfo->cipher = cipher;
     rootInfo->volumeKey = volumeKey;
-    rootInfo->root = Ptr<DirNode>( new DirNode( rootDir, dirNodeConfig ));
+    rootInfo->root = shared_ptr<DirNode>( 
+	    new DirNode( ctx, rootDir, dirNodeConfig ));
 
     return rootInfo;
 }
 
 void showFSInfo( const EncFSConfig &config )
 {
-    Ptr<Cipher> cipher = Cipher::New( config.cipherIface, -1 );
+    shared_ptr<Cipher> cipher = Cipher::New( config.cipherIface, -1 );
     {
 	cout << autosprintf(
 		// xgroup(diag)
@@ -1024,7 +1130,7 @@ void showFSInfo( const EncFSConfig &config )
 		config.cipherIface.name().c_str(), config.cipherIface.current(),
 		config.cipherIface.revision(), config.cipherIface.age());
 	// check if we support this interface..
-	if(cipher.isNull())
+	if(!cipher)
 	    cout << _(" (NOT supported)\n");
 	else
 	{
@@ -1046,9 +1152,9 @@ void showFSInfo( const EncFSConfig &config )
 		config.nameIface.revision(), config.nameIface.age());
 	    
 	// check if we support the filename encoding interface..
-	Ptr<NameIO> nameCoder = NameIO::New( config.nameIface, 
-    		cipher, OpaqueValue() );
-	if(nameCoder.isNull())
+	shared_ptr<NameIO> nameCoder = NameIO::New( config.nameIface, 
+    		cipher, CipherKey() );
+	if(!nameCoder)
 	{
 	    // xgroup(diag)
 	    cout << _(" (NOT supported)\n");
@@ -1067,7 +1173,7 @@ void showFSInfo( const EncFSConfig &config )
     {
 	cout << autosprintf(_("Key Size: %i bits"), config.keySize);
 	cipher = Cipher::New( config.cipherIface, config.keySize );
-	if(cipher.isNull())
+	if(!cipher)
 	{
 	    // xgroup(diag)
 	    cout << _(" (NOT supported)\n");
@@ -1114,25 +1220,30 @@ void showFSInfo( const EncFSConfig &config )
 	// xgroup(diag)
 	cout << _("File data IV is chained to filename IV.\n");
     }
+    if(config.allowHoles)
+    {
+	// xgroup(diag)
+	cout << _("File holes passed through to ciphertext.\n");
+    }
     cout << "\n";
 }
     
-CipherKey getUserKey( const Ptr<Cipher> &cipher, bool useStdin )
+CipherKey getUserKey( const shared_ptr<Cipher> &cipher, bool useStdin )
 {
-    const int MaxPassBuf = 64;
     char passBuf[MaxPassBuf];
     char *res;
 
     if( useStdin )
     {
-	res = fgets( passBuf, sizeof(passBuf) - 1, stdin );
+	res = fgets( passBuf, sizeof(passBuf), stdin );
 	// Kill the trailing newline.
-	passBuf[ strlen(passBuf)-1 ] = '\0';
+	if(passBuf[ strlen(passBuf)-1 ] == '\n')
+	    passBuf[ strlen(passBuf)-1 ] = '\0';
     } else
     {
 	// xgroup(common)
 	res = readpassphrase( _("EncFS Password: "),
-		passBuf, sizeof(passBuf)-1, RPP_ECHO_OFF );
+		passBuf, sizeof(passBuf), RPP_ECHO_OFF );
     }
 
     CipherKey userKey;
@@ -1148,7 +1259,7 @@ CipherKey getUserKey( const Ptr<Cipher> &cipher, bool useStdin )
 
 std::string readPassword( int FD )
 {
-    char buffer[2048];
+    char buffer[1024];
     string result;
 
     while(1)
@@ -1172,8 +1283,8 @@ std::string readPassword( int FD )
     return result;
 }
 
-CipherKey getUserKey( const std::string &passProg, const Ptr<Cipher> &cipher,
-	const std::string &rootDir )
+CipherKey getUserKey( const std::string &passProg, 
+	const shared_ptr<Cipher> &cipher, const std::string &rootDir )
 {
     // have a child process run the command and get the result back to us.
     int fds[2], pid;
@@ -1199,9 +1310,11 @@ CipherKey getUserKey( const std::string &passProg, const Ptr<Cipher> &cipher,
 
     if(pid == 0)
     {
-	char *argv[2];
-	argv[0] = const_cast<char*>(passProg.c_str());
-	argv[1] = 0;
+	const char *argv[4];
+	argv[0] = "/bin/sh";
+	argv[1] = "-c";
+	argv[2] = passProg.c_str();
+	argv[3] = 0;
 
 	// child process.. run the command and send output to fds[0]
 	close(fds[1]); // we don't use the other half..
@@ -1229,7 +1342,7 @@ CipherKey getUserKey( const std::string &passProg, const Ptr<Cipher> &cipher,
 	snprintf(tmpBuf, sizeof(tmpBuf)-1, "%i", stdErrCopy);
 	setenv(ENCFS_ENV_STDERR, tmpBuf, 1);
 
-	execvp( argv[0], argv ); // returns only on error..
+	execvp( argv[0], (char * const *)argv ); // returns only on error..
 
 	perror(_("Internal error: failed to exec program"));
 	exit(1);
@@ -1250,9 +1363,8 @@ CipherKey getUserKey( const std::string &passProg, const Ptr<Cipher> &cipher,
     return result;
 }
 
-CipherKey getNewUserKey( const Ptr<Cipher> &cipher )
+CipherKey getNewUserKey( const shared_ptr<Cipher> &cipher )
 {
-    const int MaxPassBuf = 64;
     CipherKey userKey;
     char passBuf[MaxPassBuf];
     char passBuf2[MaxPassBuf];
@@ -1276,21 +1388,33 @@ CipherKey getNewUserKey( const Ptr<Cipher> &cipher )
 
 	memset( passBuf, 0, sizeof(passBuf) );
 	memset( passBuf2, 0, sizeof(passBuf2) );
-    } while( userKey.isNull() );
+    } while( !userKey );
 
     return userKey;
 }
 
-RootPtr initFS( const EncFS_Opts *opts )
+RootPtr initFS( EncFS_Context *ctx, const shared_ptr<EncFS_Opts> &opts )
 {
     RootPtr rootInfo;
     EncFSConfig config;
 
     if(readConfig( opts->rootDir, &config ) != Config_None)
     {
-	// first, instanciate the cipher.
-	Ptr<Cipher> cipher = Cipher::New( config.cipherIface, config.keySize );
-	if(cipher.isNull())
+	if(opts->reverseEncryption)
+	{
+	    if (config.blockMACBytes != 0 || config.blockMACRandBytes != 0
+		|| config.uniqueIV == true || config.externalIVChaining == true
+		|| config.chainedNameIV == true )
+	    {  
+		cout << _("The configuration loaded is not compatible with --reverse\n");
+		return rootInfo;
+	    }
+	}
+
+// first, instanciate the cipher.
+	shared_ptr<Cipher> cipher = 
+	    Cipher::New( config.cipherIface, config.keySize );
+	if(!cipher)
 	{
 	    rError(_("Unable to find cipher %s, version %i:%i:%i"),
 		    config.cipherIface.name().c_str(),
@@ -1311,7 +1435,7 @@ RootPtr initFS( const EncFS_Opts *opts )
 	    userKey = getUserKey( opts->passwordProgram, 
 		    cipher, opts->rootDir );
 
-	if(userKey.isNull())
+	if(!userKey)
 	    return rootInfo;
 
 	// no funny stuff
@@ -1323,16 +1447,16 @@ RootPtr initFS( const EncFS_Opts *opts )
 		(unsigned char*)config.keyData.data(), userKey, opts->checkKey);
 	userKey.reset();
 
-	if(volumeKey.isNull())
+	if(!volumeKey)
 	{
 	    // xgroup(diag)
 	    cout << _("Error decoding volume key, password incorrect\n");
 	    return rootInfo;
 	}
 
-	Ptr<NameIO> nameCoder = NameIO::New( config.nameIface, 
+	shared_ptr<NameIO> nameCoder = NameIO::New( config.nameIface, 
 		cipher, volumeKey );
-	if(nameCoder.isNull())
+	if(!nameCoder)
 	{
 	    rError(_("Unable to find nameio interface %s, version %i:%i:%i"),
 		    config.nameIface.name().c_str(),
@@ -1346,8 +1470,9 @@ RootPtr initFS( const EncFS_Opts *opts )
 	}
    
 	nameCoder->setChainedNameIV( config.chainedNameIV );
+	nameCoder->setReverseEncryption( opts->reverseEncryption );
 
-	Ptr<DirNode::Config> dirNodeConfig (new DirNode::Config);
+	shared_ptr<DirNode::Config> dirNodeConfig (new DirNode::Config);
 	dirNodeConfig->cipher = cipher;
 	dirNodeConfig->key = volumeKey;
 	dirNodeConfig->nameCoding = nameCoder;
@@ -1359,22 +1484,40 @@ RootPtr initFS( const EncFS_Opts *opts )
 	dirNodeConfig->uniqueIV = config.uniqueIV;
 	dirNodeConfig->externalIVChaining = config.externalIVChaining;
 	dirNodeConfig->forceDecode = opts->forceDecode;
+	dirNodeConfig->reverseEncryption = opts->reverseEncryption;
 
 	rootInfo = RootPtr( new EncFS_Root );
 	rootInfo->cipher = cipher;
 	rootInfo->volumeKey = volumeKey;
-	rootInfo->root = Ptr<DirNode>( 
-		new DirNode( opts->rootDir, dirNodeConfig ));
+	rootInfo->root = shared_ptr<DirNode>( 
+		new DirNode( ctx, opts->rootDir, dirNodeConfig ));
     } else
     {
 	if(opts->createIfNotFound)
 	{
 	    // creating a new encrypted filesystem
-	    rootInfo = createV5Config( opts->rootDir, opts->idleTracking,
-		    opts->forceDecode, opts->passwordProgram, opts->useStdin );
+	    rootInfo = createV6Config( ctx, opts->rootDir, opts->idleTracking,
+		    opts->forceDecode, opts->passwordProgram, opts->useStdin,
+		    opts->reverseEncryption );
 	}
     }
 	
     return rootInfo;
 }
 
+int remountFS(EncFS_Context *ctx)
+{
+    rDebug("Attempting to reinitialize filesystem");
+
+    RootPtr rootInfo = initFS( ctx, ctx->opts );
+    if(rootInfo)
+    {
+	ctx->setRoot(rootInfo->root);
+	return 0;
+    } else
+    {
+	rInfo(_("Remount failed"));
+	return -EACCES;
+    }
+}
+
