@@ -21,10 +21,7 @@
 #include "encfs.h"
 #include "Interface.h"
 #include "DirNode.h"
-
-#include "Ptr.h"
-
-using rel::Ptr;
+#include "CipherKey.h"
 
 // true if the path points to an existing node (of any type)
 bool fileExists( const char *fileName );
@@ -61,6 +58,18 @@ struct EncFSConfig
     bool externalIVChaining; // IV seeding by filename IV chaining
 
     bool chainedNameIV; // filename IV chaining
+    bool allowHoles; // allow holes in files (implicit zero blocks)
+
+    EncFSConfig()
+    {
+        subVersion = 0;
+        blockMACBytes = 0;
+        blockMACRandBytes = 0;
+        uniqueIV = false;
+        externalIVChaining = false;
+        chainedNameIV = false;
+        allowHoles = false;
+    }
 };
 
 enum ConfigType
@@ -69,23 +78,23 @@ enum ConfigType
     Config_Prehistoric,
     Config_V3,
     Config_V4,
-    Config_V5
+    Config_V5,
+    Config_V6
 };
 
 class Cipher;
-typedef rel::OpaqueValue CipherKey;
 
 struct EncFS_Root
 {
-    Ptr<Cipher> cipher;
+    boost::shared_ptr<Cipher> cipher;
     CipherKey volumeKey;
-    Ptr<DirNode> root;
+    boost::shared_ptr<DirNode> root;
 
     EncFS_Root();
     ~EncFS_Root();
 };
 
-typedef Ptr<EncFS_Root> RootPtr;
+typedef boost::shared_ptr<EncFS_Root> RootPtr;
 
 /*
     Read existing config file.  Looks for any supported configuration version.
@@ -105,6 +114,7 @@ struct EncFS_Opts
     std::string rootDir;
     bool createIfNotFound;  // create filesystem if not found
     bool idleTracking; // turn on idle monitoring of filesystem
+    bool mountOnDemand; // mounting on-demand
 
     bool checkKey;  // check crypto key decoding
     bool forceDecode; // force decode on MAC block failures
@@ -114,33 +124,33 @@ struct EncFS_Opts
 
     bool ownerCreate; // set owner of new files to caller
 
+    bool reverseEncryption; // Reverse encryption
     EncFS_Opts()
     {
 	createIfNotFound = true;
 	idleTracking = false;
+	mountOnDemand = false;
 	checkKey = true;
 	forceDecode = false;
 	useStdin = false;
 	ownerCreate = false;
+	reverseEncryption = false;
     }
 };
 
+class EncFS_Context;
 
-RootPtr initFS( const EncFS_Opts *opts );
+RootPtr initFS( EncFS_Context *ctx, const shared_ptr<EncFS_Opts> &opts );
 
-RootPtr createV5Config( const std::string &rootDir, 
+RootPtr createV6Config( EncFS_Context *ctx, const std::string &rootDir, 
 	bool enableIdleTracking,
 	bool forceDecode,
-	const std::string &passwordProgram );
+	const std::string &passwordProgram, bool reverseEncryption,
+        bool allowHoles );
 
 
 void showFSInfo( const EncFSConfig &config );
 
-// Read specifically a version 4 configuration file. 
-bool readV3Config( const char *configFile, EncFSConfig *config,
-	struct ConfigInfo *);
-bool writeV3Config( const char *configFile, EncFSConfig *config);
-
 bool readV4Config( const char *configFile, EncFSConfig *config,
 	struct ConfigInfo *);
 bool writeV4Config( const char *configFile, EncFSConfig *config);
@@ -149,12 +159,16 @@ bool readV5Config( const char *configFile, EncFSConfig *config,
 	struct ConfigInfo *);
 bool writeV5Config( const char *configFile, EncFSConfig *config);
 
+bool readV6Config( const char *configFile, EncFSConfig *config,
+	struct ConfigInfo *);
+bool writeV6Config( const char *configFile, EncFSConfig *config);
 
 
-CipherKey getUserKey(const Ptr<Cipher> &cipher, bool useStdin);
+
+CipherKey getUserKey(const boost::shared_ptr<Cipher> &cipher, bool useStdin);
 CipherKey getUserKey(const std::string &passwordProgram, 
-	             const Ptr<Cipher> &cipher,
+	             const boost::shared_ptr<Cipher> &cipher,
 		     const std::string &rootDir );
-CipherKey getNewUserKey(const Ptr<Cipher> &cipher);
+CipherKey getNewUserKey(const boost::shared_ptr<Cipher> &cipher);
 
 #endif
