@@ -103,10 +103,10 @@ bool NameIO::Register( const char *name, const char *description,
     return true;
 }
 
-Ptr<NameIO> NameIO::New( const string &name, const Ptr<Cipher> &cipher,
-	const CipherKey &key)
+shared_ptr<NameIO> NameIO::New( const string &name, 
+	const shared_ptr<Cipher> &cipher, const CipherKey &key)
 {
-    Ptr<NameIO> result;
+    shared_ptr<NameIO> result;
     if(gNameIOMap)
     {
 	NameIOMap_t::const_iterator it = gNameIOMap->find( name );
@@ -119,10 +119,10 @@ Ptr<NameIO> NameIO::New( const string &name, const Ptr<Cipher> &cipher,
     return result;
 }
 
-Ptr<NameIO> NameIO::New( const Interface &iface, const Ptr<Cipher> &cipher,
-	const CipherKey &key )
+shared_ptr<NameIO> NameIO::New( const Interface &iface, 
+	const shared_ptr<Cipher> &cipher, const CipherKey &key )
 {
-    Ptr<NameIO> result;
+    shared_ptr<NameIO> result;
     if(gNameIOMap)
     {
 	NameIOMap_t::const_iterator it;
@@ -143,7 +143,7 @@ Ptr<NameIO> NameIO::New( const Interface &iface, const Ptr<Cipher> &cipher,
 
 
 NameIO::NameIO()
-    : chainedNameIV( false )
+    : chainedNameIV( false ), reverseEncryption( false )
 {
 }
 
@@ -161,6 +161,17 @@ bool NameIO::getChainedNameIV() const
     return chainedNameIV;
 }
 
+void NameIO::setReverseEncryption( bool enable )
+{
+    reverseEncryption = enable;
+}
+
+bool NameIO::getReverseEncryption() const
+{
+    return reverseEncryption;
+}
+
+
 std::string NameIO::recodePath( const char *path, 
 	int (NameIO::*_length)(int) const,
 	int (NameIO::*_code)(const char*, int, uint64_t *, char*) const,
@@ -215,20 +226,16 @@ std::string NameIO::recodePath( const char *path,
 std::string NameIO::encodePath( const char *plaintextPath ) const
 {
     uint64_t iv = 0;
-    uint64_t *ivptr = chainedNameIV ? &iv : (uint64_t*)0;
-    return recodePath( plaintextPath, 
-	    &NameIO::maxEncodedNameLen, &NameIO::encodeName, ivptr);
+    return encodePath( plaintextPath, &iv);
 }
 
 std::string NameIO::decodePath( const char *cipherPath ) const
 {
     uint64_t iv = 0;
-    uint64_t *ivptr = chainedNameIV ? &iv : (uint64_t*)0;
-    return recodePath( cipherPath, 
-	    &NameIO::maxDecodedNameLen, &NameIO::decodeName, ivptr);
+    return decodePath( cipherPath, &iv );
 }
 
-std::string NameIO::encodePath( const char *plaintextPath, uint64_t *iv ) const
+std::string NameIO::_encodePath( const char *plaintextPath, uint64_t *iv ) const
 {
     // if chaining is not enabled, then the iv pointer is not used..
     if(!chainedNameIV)
@@ -237,7 +244,7 @@ std::string NameIO::encodePath( const char *plaintextPath, uint64_t *iv ) const
 	    &NameIO::maxEncodedNameLen, &NameIO::encodeName, iv);
 }
 
-std::string NameIO::decodePath( const char *cipherPath, uint64_t *iv ) const
+std::string NameIO::_decodePath( const char *cipherPath, uint64_t *iv ) const
 {
     // if chaining is not enabled, then the iv pointer is not used..
     if(!chainedNameIV)
@@ -246,6 +253,21 @@ std::string NameIO::decodePath( const char *cipherPath, uint64_t *iv ) const
 	    &NameIO::maxDecodedNameLen, &NameIO::decodeName, iv);
 }
 
+std::string NameIO::encodePath( const char *path, uint64_t *iv ) const
+{
+    return getReverseEncryption() ? 
+	_decodePath( path, iv ) :
+	_encodePath( path, iv );
+} 
+
+std::string NameIO::decodePath( const char *path, uint64_t *iv ) const
+{
+    return getReverseEncryption() ? 
+	_encodePath( path, iv ) :
+	_decodePath( path, iv );
+} 
+
+
 int NameIO::encodeName( const char *input, int length, char *output ) const
 {
     return encodeName( input, length, (uint64_t*)0, output );
@@ -256,7 +278,7 @@ int NameIO::decodeName( const char *input, int length, char *output ) const
     return decodeName( input, length, (uint64_t*)0, output );
 }
 
-std::string NameIO::encodeName( const char *plaintextName, int length ) const
+std::string NameIO::_encodeName( const char *plaintextName, int length ) const
 {
     int approxLen = maxEncodedNameLen( length );
 
@@ -275,7 +297,7 @@ std::string NameIO::encodeName( const char *plaintextName, int length ) const
     return result;
 }
 
-std::string NameIO::decodeName( const char *encodedName, int length ) const
+std::string NameIO::_decodeName( const char *encodedName, int length ) const
 {
     int approxLen = maxDecodedNameLen( length );
 
@@ -294,3 +316,33 @@ std::string NameIO::decodeName( const char *encodedName, int length ) const
     return result;
 }
 
+std::string NameIO::encodeName( const char *path, int length ) const
+{
+   return getReverseEncryption() ?
+	_decodeName( path, length ) :
+	_encodeName( path, length );
+}
+
+std::string NameIO::decodeName( const char *path, int length ) const
+{
+   return getReverseEncryption() ?
+	_encodeName( path, length ) :
+	_decodeName( path, length );
+}
+/*
+int NameIO::encodeName( const char *path, int length, 
+	char *output ) const
+{
+   return getReverseEncryption() ?
+	_decodeName( path, length, output ) :
+	_encodeName( path, length, output );
+}
+
+int NameIO::decodeName( const char *path, int length,
+	 char *output ) const
+{
+   return getReverseEncryption() ?
+	_encodeName( path, length, output ) :
+	_decodeName( path, length, output );
+}
+*/
