@@ -20,21 +20,26 @@
 #include "Range.h"
 #include "Interface.h"
 
+#include <boost/shared_ptr.hpp>
 #include <rlog/rlog.h>
 
+#include <string.h>
+
 using namespace std;
 using namespace rel;
 using namespace rlog;
+using boost::shared_ptr;
+using boost::dynamic_pointer_cast;
 
 
 static Interface NullInterface( "nullCipher", 1, 0, 0 );
 static Range NullKeyRange(0);
 static Range NullBlockRange(1,4096,1);
 
-static Ptr<Cipher> NewNullCipher(const Interface &iface, int keyLen)
+static shared_ptr<Cipher> NewNullCipher(const Interface &iface, int keyLen)
 {
     (void)keyLen;
-    return Ptr<Cipher>( new NullCipher( iface ) );
+    return shared_ptr<Cipher>( new NullCipher( iface ) );
 }
 
 const bool HiddenCipher = true;
@@ -44,10 +49,25 @@ static bool NullCipher_registered = Cipher::Register("Null",
 	NullInterface, NullKeyRange, NullBlockRange, NewNullCipher,
 	HiddenCipher);
 
-struct NullKey
+class NullKey : public AbstractCipherKey
 {
+public:
+    NullKey() {}
+    virtual ~NullKey() {}
+};
 
-} gNullKey;
+class NullDestructor
+{
+public:
+    NullDestructor() {}
+    NullDestructor(const NullDestructor &) {}
+    ~NullDestructor() {}
+
+    NullDestructor &operator = (const NullDestructor &){ return *this; }
+    void operator ()(NullKey *&) {}
+};
+
+shared_ptr<AbstractCipherKey> gNullKey( new NullKey(), NullDestructor() );
 
 NullCipher::NullCipher(const Interface &iface_)
 {
@@ -65,14 +85,12 @@ Interface NullCipher::interface() const
 
 CipherKey NullCipher::newKey(const char *, int )
 {
-    // no destructor..
-    return Ptr<NullKey>( &gNullKey, (void*)NULL );
+    return gNullKey;
 }
 
 CipherKey NullCipher::newRandomKey()
 {
-    // no destructor..
-    return Ptr<NullKey>( &gNullKey, (void*)NULL );
+    return gNullKey;
 }
 
 void NullCipher::randomize( unsigned char *buf, int len ) const
@@ -89,8 +107,7 @@ uint64_t NullCipher::MAC_64(const unsigned char *, int ,
 CipherKey NullCipher::readKey( const unsigned char *, 
 	const CipherKey &, bool)
 {
-    // no destructor..
-    return Ptr<NullKey>( &gNullKey, (void*)NULL );
+    return gNullKey;
 }
 
 void NullCipher::writeKey(const CipherKey &, unsigned char *, 
@@ -101,8 +118,8 @@ void NullCipher::writeKey(const CipherKey &, unsigned char *,
 bool NullCipher::compareKey(const CipherKey &A_, 
 	const CipherKey &B_) const
 {
-    Ptr<NullKey> A = A_;
-    Ptr<NullKey> B = B_;
+    shared_ptr<NullKey> A = dynamic_pointer_cast<NullKey>(A_);
+    shared_ptr<NullKey> B = dynamic_pointer_cast<NullKey>(B_);
     return A.get() == B.get();
 }
 
