@@ -44,6 +44,8 @@ using namespace std;
 using namespace rel;
 using namespace rlog;
 
+using boost::shared_ptr;
+using boost::dynamic_pointer_cast;
 
 const int MAX_KEYLENGTH = 32; // in bytes (256 bit)
 const int MAX_IVLENGTH = 16;
@@ -136,7 +138,7 @@ static Interface AESInterface( "ssl/aes", 2, 1, 1 );
 static Range BFKeyRange(128,256,32);
 static Range BFBlockRange(64,4096,8);
 
-static Ptr<Cipher> NewBFCipher( const Interface &iface, int keyLen )
+static shared_ptr<Cipher> NewBFCipher( const Interface &iface, int keyLen )
 {
     if( keyLen <= 0 )
 	keyLen = 160;
@@ -146,7 +148,7 @@ static Ptr<Cipher> NewBFCipher( const Interface &iface, int keyLen )
     const EVP_CIPHER *blockCipher = EVP_bf_cbc();
     const EVP_CIPHER *streamCipher = EVP_bf_cfb();
 
-    return Ptr<Cipher>( new SSL_Cipher(iface, BlowfishInterface,
+    return shared_ptr<Cipher>( new SSL_Cipher(iface, BlowfishInterface,
 	    blockCipher, streamCipher, keyLen / 8) );
 }
 
@@ -162,7 +164,7 @@ static bool BF_Cipher_registered = Cipher::Register("Blowfish",
 static Range AESKeyRange(128,256,64);
 static Range AESBlockRange(64,4096,16);
 
-static Ptr<Cipher> NewAESCipher( const Interface &iface, int keyLen )
+static shared_ptr<Cipher> NewAESCipher( const Interface &iface, int keyLen )
 {
     if( keyLen <= 0 )
 	keyLen = 192;
@@ -191,7 +193,7 @@ static Ptr<Cipher> NewAESCipher( const Interface &iface, int keyLen )
 	break;
     }
     
-    return Ptr<Cipher>( new SSL_Cipher(iface, AESInterface, 
+    return shared_ptr<Cipher>( new SSL_Cipher(iface, AESInterface, 
 		blockCipher, streamCipher, keyLen / 8) );
 }
 
@@ -200,8 +202,9 @@ static bool AES_Cipher_registered = Cipher::Register("AES",
 	AESInterface, AESKeyRange, AESBlockRange, NewAESCipher);
 #endif
 
-struct SSLKey
+class SSLKey : public AbstractCipherKey
 {
+public:
     pthread_mutex_t mutex;
     
     unsigned int keySize; // in bytes
@@ -256,16 +259,16 @@ SSLKey::~SSLKey()
 }
 
 
-inline unsigned char* KeyData( const Ptr<SSLKey> &key )
+inline unsigned char* KeyData( const shared_ptr<SSLKey> &key )
 {
     return key->buffer;
 }
-inline unsigned char* IVData( const Ptr<SSLKey> &key )
+inline unsigned char* IVData( const shared_ptr<SSLKey> &key )
 {
     return key->buffer + key->keySize;
 }
 
-void initKey(const Ptr<SSLKey> &key, const EVP_CIPHER *_blockCipher,
+void initKey(const shared_ptr<SSLKey> &key, const EVP_CIPHER *_blockCipher,
 	const EVP_CIPHER *_streamCipher, int _keySize)
 {
     Lock lock( key->mutex );
@@ -357,7 +360,7 @@ CipherKey SSL_Cipher::newKey(const char *password, int passwdLength)
 	return CipherKey();
     }
    
-    Ptr<SSLKey> key( new SSLKey( _keySize, _ivLength) );
+    shared_ptr<SSLKey> key( new SSLKey( _keySize, _ivLength) );
     
     int bytes = 0;
     if( iface.current() > 1 )
@@ -412,7 +415,7 @@ CipherKey SSL_Cipher::newRandomKey()
 	}
     }
 
-    Ptr<SSLKey> key( new SSLKey( _keySize, _ivLength) );
+    shared_ptr<SSLKey> key( new SSLKey( _keySize, _ivLength) );
 
     // doesn't need to be versioned, because a random key is a random key..
     // Doesn't need to be reproducable..
@@ -485,7 +488,7 @@ void SSL_Cipher::randomize( unsigned char *buf, int len ) const
 uint64_t SSL_Cipher::MAC_64( const unsigned char *data, int len,
 	const CipherKey &key, uint64_t *chainedIV ) const
 {
-    Ptr<SSLKey> mk = key;
+    shared_ptr<SSLKey> mk = dynamic_pointer_cast<SSLKey>(key);
     uint64_t tmp = _checksum_64( mk.get(), data, len, chainedIV );
 
     if(chainedIV)
@@ -497,7 +500,7 @@ uint64_t SSL_Cipher::MAC_64( const unsigned char *data, int len,
 CipherKey SSL_Cipher::readKey(const unsigned char *data, 
 	const CipherKey &masterKey, bool checkKey)
 {
-    Ptr<SSLKey> mk = masterKey;
+    shared_ptr<SSLKey> mk = dynamic_pointer_cast<SSLKey>(masterKey);
     rAssert(mk->keySize == _keySize);
     
     unsigned char tmpBuf[ MAX_KEYLENGTH + MAX_IVLENGTH ];
@@ -520,7 +523,7 @@ CipherKey SSL_Cipher::readKey(const unsigned char *data,
 	return CipherKey();
     }
     
-    Ptr<SSLKey> key( new SSLKey( _keySize, _ivLength) );
+    shared_ptr<SSLKey> key( new SSLKey( _keySize, _ivLength) );
     
     memcpy( key->buffer, tmpBuf, _keySize + _ivLength );
     memset( tmpBuf, 0, sizeof(tmpBuf) );
@@ -533,11 +536,11 @@ CipherKey SSL_Cipher::readKey(const unsigned char *data,
 void SSL_Cipher::writeKey(const CipherKey &ckey, unsigned char *data, 
 	const CipherKey &masterKey)
 {
-    Ptr<SSLKey> key = ckey;
+    shared_ptr<SSLKey> key = dynamic_pointer_cast<SSLKey>(ckey);
     rAssert(key->keySize == _keySize);
     rAssert(key->ivLength == _ivLength);
 
-    Ptr<SSLKey> mk = masterKey;
+    shared_ptr<SSLKey> mk = dynamic_pointer_cast<SSLKey>(masterKey);
     rAssert(mk->keySize == _keySize);
     rAssert(mk->ivLength == _ivLength);
 
@@ -563,8 +566,8 @@ void SSL_Cipher::writeKey(const CipherKey &ckey, unsigned char *data,
 
 bool SSL_Cipher::compareKey( const CipherKey &A, const CipherKey &B) const
 {
-    Ptr<SSLKey> key1 = A;
-    Ptr<SSLKey> key2 = B;
+    shared_ptr<SSLKey> key1 = dynamic_pointer_cast<SSLKey>(A);
+    shared_ptr<SSLKey> key2 = dynamic_pointer_cast<SSLKey>(B);
 
     rAssert(key1->keySize == _keySize);
     rAssert(key2->keySize == _keySize);
@@ -591,7 +594,7 @@ int SSL_Cipher::cipherBlockSize() const
 }
 
 void SSL_Cipher::setIVec( unsigned char *ivec, unsigned int seed,
-	const Ptr<SSLKey> &key) const
+	const shared_ptr<SSLKey> &key) const
 {
     /* These multiplication constants chosen as they represent (non optimal)
        Golumb rulers, the idea being to spread around the information in the
@@ -665,7 +668,7 @@ bool SSL_Cipher::streamEncode(unsigned char *buf, int size,
 	uint64_t iv64, const CipherKey &ckey) const
 {
     rAssert( size > 0 );
-    Ptr<SSLKey> key = ckey;
+    shared_ptr<SSLKey> key = dynamic_pointer_cast<SSLKey>(ckey);
     rAssert(key->keySize == _keySize);
     rAssert(key->ivLength == _ivLength);
 
@@ -703,7 +706,7 @@ bool SSL_Cipher::streamDecode(unsigned char *buf, int size,
 	uint64_t iv64, const CipherKey &ckey) const
 {
     rAssert( size > 0 );
-    Ptr<SSLKey> key = ckey;
+    shared_ptr<SSLKey> key = dynamic_pointer_cast<SSLKey>(ckey);
     rAssert(key->keySize == _keySize);
     rAssert(key->ivLength == _ivLength);
 
@@ -742,7 +745,7 @@ bool SSL_Cipher::blockEncode(unsigned char *buf, int size,
 	uint64_t iv64, const CipherKey &ckey ) const
 {
     rAssert( size > 0 );
-    Ptr<SSLKey> key = ckey;
+    shared_ptr<SSLKey> key = dynamic_pointer_cast<SSLKey>(ckey);
     rAssert(key->keySize == _keySize);
     rAssert(key->ivLength == _ivLength);
     
@@ -776,7 +779,7 @@ bool SSL_Cipher::blockDecode(unsigned char *buf, int size,
 	uint64_t iv64, const CipherKey &ckey ) const
 {
     rAssert( size > 0 );
-    Ptr<SSLKey> key = ckey;
+    shared_ptr<SSLKey> key = dynamic_pointer_cast<SSLKey>(ckey);
     rAssert(key->keySize == _keySize);
     rAssert(key->ivLength == _ivLength);
 
