@@ -28,6 +28,8 @@ struct evp_cipher_st;
 typedef struct evp_cipher_st EVP_CIPHER;
 #endif
 
+using boost::shared_ptr;
+
 /*
     Implements Cipher interface for OpenSSL's ciphers.
 
@@ -131,7 +133,7 @@ public:
     static bool Enabled();
 private:
     void setIVec( unsigned char *ivec, unsigned int seed, 
-	    const Ptr<SSLKey> &key ) const;
+	    const shared_ptr<SSLKey> &key ) const;
 };
 
 
