@@ -28,10 +28,10 @@
 using namespace rel;
 using namespace std;
 
-static Ptr<NameIO> NewStreamNameIO( const Interface &iface,
-	const Ptr<Cipher> &cipher, const CipherKey &key)
+static shared_ptr<NameIO> NewStreamNameIO( const Interface &iface,
+	const shared_ptr<Cipher> &cipher, const CipherKey &key)
 {
-    return Ptr<NameIO>( new StreamNameIO( iface, cipher, key ) );
+    return shared_ptr<NameIO>( new StreamNameIO( iface, cipher, key ) );
 }
 
 static bool StreamIO_registered = NameIO::Register("Stream",
@@ -68,7 +68,7 @@ Interface StreamNameIO::CurrentInterface()
 }
 
 StreamNameIO::StreamNameIO( const rel::Interface &iface,
-	const rel::Ptr<Cipher> &cipher, 
+	const shared_ptr<Cipher> &cipher, 
 	const CipherKey &key )
     : _interface( iface.current() )
     , _cipher( cipher )
@@ -186,6 +186,8 @@ int StreamNameIO::decodeName( const char *encodedName, int length,
     // compute MAC to check with stored value
     unsigned int mac2 = _cipher->MAC_16((const unsigned char *)plaintextName, 
 	    decodedStreamLen, _key, iv);
+    
+    BUFFER_RESET( tmpBuf );
     if(mac2 != mac)
     {
 	rDebug("checksum mismatch: expected %u, got %u", mac, mac2);
@@ -193,8 +195,6 @@ int StreamNameIO::decodeName( const char *encodedName, int length,
 	throw ERROR( "checksum mismatch in filename decode" );
     }
 
-    BUFFER_RESET( tmpBuf );
-
     return decodedStreamLen;
 }
 
