@@ -24,6 +24,8 @@
 #include "FileUtils.h"
 #include "Cipher.h"
 
+#include "Context.h"
+
 #include <rlog/rlog.h>
 #include <rlog/StdioNode.h>
 #include <rlog/RLogChannel.h>
@@ -40,6 +42,8 @@
 
 #include "i18n.h"
 
+#include <boost/scoped_array.hpp>
+
 #ifdef HAVE_SSL
 #define NO_DES
 #include <openssl/ssl.h>
@@ -48,6 +52,7 @@
 using namespace rlog;
 using namespace std;
 using namespace gnu;
+using namespace boost;
 
 
 static int showInfo( int argc, char **argv );
@@ -195,6 +200,12 @@ static int showInfo( int argc, char **argv )
 	    "created by %s (revision %i)\n"), config.creator.c_str(),
 		config.subVersion);
 	break;
+    case Config_V6:
+	// xgroup(diag)
+	cout << "\n" << autosprintf(_("Version 6 configuration; "
+	    "created by %s (revision %i)\n"), config.creator.c_str(),
+		config.subVersion);
+	break;
     }
 
     showFSInfo( config );
@@ -209,14 +220,14 @@ static RootPtr initRootInfo(const char* crootDir)
 
     if(checkDir( rootDir ))
     {
-	EncFS_Opts opts;
-	opts.rootDir = rootDir;
-	opts.createIfNotFound = false;
-	opts.checkKey = false;
-	result = initFS( &opts );
+	shared_ptr<EncFS_Opts> opts( new EncFS_Opts() );
+	opts->rootDir = rootDir;
+	opts->createIfNotFound = false;
+	opts->checkKey = false;
+	result = initFS( NULL, opts );
     }
 
-    if(result.isNull())
+    if(!result)
         cerr << _("Unable to initialize encrypted filesystem - check path.\n");
 
     return result;
@@ -226,7 +237,7 @@ static int cmd_decode( int argc, char **argv )
 {
     RootPtr rootInfo = initRootInfo(argv[1]);
 
-    if(rootInfo.isNull())
+    if(!rootInfo)
 	return EXIT_FAILURE;
 
     if( argc > 2 )
@@ -253,7 +264,7 @@ static int cmd_encode( int argc, char **argv )
 {
     RootPtr rootInfo = initRootInfo(argv[1]);
 
-    if(rootInfo.isNull())
+    if(!rootInfo)
 	return EXIT_FAILURE;
 
     if( argc > 2 )
@@ -282,7 +293,7 @@ static int cmd_ls( int argc, char **argv )
 
     RootPtr rootInfo = initRootInfo(argv[1]);
 
-    if(rootInfo.isNull())
+    if(!rootInfo)
 	return EXIT_FAILURE;
 
     // show files in directory
@@ -293,7 +304,7 @@ static int cmd_ls( int argc, char **argv )
 	    for(string name = dt.nextPlaintextName(); !name.empty(); 
 		    name = dt.nextPlaintextName())
 	    {
-		Ptr<FileNode> fnode = 
+		shared_ptr<FileNode> fnode = 
 		    rootInfo->root->lookupNode( name.c_str(), "encfsctl-ls" );
 		struct stat stbuf;
 		fnode->getAttr( &stbuf );
@@ -317,14 +328,15 @@ static int cmd_ls( int argc, char **argv )
 
 // apply an operation to every block in the file
 template<typename T>
-int processContents( const Ptr<EncFS_Root> &rootInfo, const char *path, T &op )
+int processContents( const shared_ptr<EncFS_Root> &rootInfo, 
+	const char *path, T &op )
 {
     int errCode = 0;
-    Ptr<FileNode> node = rootInfo->root->openNode( path, "encfsctl",
+    shared_ptr<FileNode> node = rootInfo->root->openNode( path, "encfsctl",
 	    O_RDONLY, &errCode );
-    if(node.isNull())
+    if(!node)
     {
-	cerr << "unable to open " << path << endl;
+	cerr << "unable to open " << path << "\n";
 	return errCode;
     } else
     {
@@ -338,9 +350,6 @@ int processContents( const Ptr<EncFS_Root> &rootInfo, const char *path, T &op )
 	    if(res < 0)
 		return res;
 	}
-
-	// avoid 'leak' error message..
-	rootInfo->root->release( path );
     }
     return 0;
 } 
@@ -363,7 +372,7 @@ static int cmd_cat( int argc, char **argv )
     (void)argc;
     RootPtr rootInfo = initRootInfo(argv[1]);
 
-    if(rootInfo.isNull())
+    if(!rootInfo)
 	return EXIT_FAILURE;
 
     const char *path = argv[2];
@@ -373,14 +382,40 @@ static int cmd_cat( int argc, char **argv )
     return errCode;
 }
 
-static int copyContents(const Ptr<EncFS_Root> &rootInfo, 
+static int copyLink(const struct stat &stBuf, 
+        const shared_ptr<EncFS_Root> &rootInfo,
+        const string &cpath, const string &destName )
+{
+    scoped_array<char> buf(new char[stBuf.st_size+1]);
+    int res = ::readlink( cpath.c_str(), buf.get(), stBuf.st_size );
+    if(res == -1)
+    {
+        cerr << "unable to readlink of " << cpath << "\n";
+        return EXIT_FAILURE;
+    }
+
+    buf[res] = '\0';
+    string decodedLink = rootInfo->root->plainPath(buf.get());
+
+    res = ::symlink( decodedLink.c_str(), destName.c_str() );
+    if(res == -1)
+    {
+        cerr << "unable to create symlink for " << cpath 
+            << " to " << decodedLink << "\n";
+    }
+
+    return EXIT_SUCCESS;
+}
+
+static int copyContents(const shared_ptr<EncFS_Root> &rootInfo, 
                         const char* encfsName, const char* targetName)
 {
-    Ptr<FileNode> node = rootInfo->root->lookupNode( encfsName, "encfsctl" );
+    shared_ptr<FileNode> node = 
+	rootInfo->root->lookupNode( encfsName, "encfsctl" );
 
-    if(node.isNull())
+    if(!node)
     {
-        cerr << "unable to open " << encfsName << endl;
+        cerr << "unable to open " << encfsName << "\n";
         return EXIT_FAILURE;
     } else
     {
@@ -396,7 +431,7 @@ static int copyContents(const Ptr<EncFS_Root> &rootInfo,
 
             if(readlink (d.c_str(), linkContents, PATH_MAX + 1) <= 0)
             {
-                cerr << "unable to read link " << encfsName << endl;
+                cerr << "unable to read link " << encfsName << "\n";
                 return EXIT_FAILURE;
             }
             symlink(rootInfo->root->plainPath(linkContents).c_str(), 
@@ -412,14 +447,27 @@ static int copyContents(const Ptr<EncFS_Root> &rootInfo,
     return EXIT_SUCCESS;
 }
 
-static int traverseDirs(const Ptr<EncFS_Root> &rootInfo, 
+static bool endsWith(const string &str, char ch)
+{
+    if(str.empty())
+	return false;
+    else
+	return str[str.length()-1] == ch;
+}
+
+static int traverseDirs(const shared_ptr<EncFS_Root> &rootInfo, 
 	string volumeDir, string destDir)
 {
+    if(!endsWith(volumeDir, '/'))
+	volumeDir.append("/");
+    if(!endsWith(destDir, '/'))
+	destDir.append("/");
+
     // Lookup directory node so we can create a destination directory
     // with the same permissions
     {
         struct stat st;
-        Ptr<FileNode> dirNode = 
+        shared_ptr<FileNode> dirNode = 
             rootInfo->root->lookupNode( volumeDir.c_str(), "encfsctl" );
         if(dirNode->getAttr(&st))
             return EXIT_FAILURE;
@@ -441,17 +489,29 @@ static int traverseDirs(const Ptr<EncFS_Root> &rootInfo,
                 string cpath = rootInfo->root->cipherPath(plainPath.c_str());
                 string destName = destDir + name;
 
-                if(isDirectory(cpath.c_str()))
-                    traverseDirs(rootInfo, (plainPath + '/').c_str(), 
-                                 destName + '/');
-                else
+                int r = EXIT_SUCCESS;
+                struct stat stBuf;
+                if( !lstat( cpath.c_str(), &stBuf ))
                 {
-                    int r = copyContents(rootInfo, plainPath.c_str(), 
-                                         destName.c_str());
-
-                    if(r != EXIT_SUCCESS)
-                        return r;
+                    if( S_ISDIR( stBuf.st_mode ) )
+                    {
+                        traverseDirs(rootInfo, (plainPath + '/').c_str(), 
+                                destName + '/');
+                    } else if( S_ISLNK( stBuf.st_mode ))
+                    {
+                        r = copyLink( stBuf, rootInfo, cpath, destName );
+                    } else
+                    {
+                        r = copyContents(rootInfo, plainPath.c_str(), 
+                                destName.c_str());
+                    }
+                } else
+                {
+                    r = EXIT_FAILURE;
                 }
+                    
+                if(r != EXIT_SUCCESS)
+                    return r;
             }
         }
     }
@@ -464,7 +524,7 @@ static int cmd_export( int argc, char **argv )
 
     RootPtr rootInfo = initRootInfo(argv[1]);
 
-    if(rootInfo.isNull())
+    if(!rootInfo)
         return EXIT_FAILURE;
 
     string destDir = argv[2];
@@ -475,7 +535,7 @@ static int cmd_export( int argc, char **argv )
     return traverseDirs(rootInfo, "/", destDir);
 }
 
-int showcruft( const Ptr<EncFS_Root> &rootInfo, const char *dirName )
+int showcruft( const shared_ptr<EncFS_Root> &rootInfo, const char *dirName )
 {
     int found = 0;
     DirTraverse dt = rootInfo->root->openDir( dirName );
@@ -534,7 +594,7 @@ static int cmd_showcruft( int argc, char **argv )
 
     RootPtr rootInfo = initRootInfo(argv[1]);
 
-    if(rootInfo.isNull())
+    if(!rootInfo)
 	return EXIT_FAILURE;
 
     int filesFound = showcruft( rootInfo, "/" );
@@ -563,8 +623,9 @@ static int do_chpasswd( bool useStdin, int argc, char **argv )
     }
 
     // instanciate proper cipher
-    Ptr<Cipher> cipher = Cipher::New( config.cipherIface, config.keySize );
-    if(cipher.isNull())
+    shared_ptr<Cipher> cipher = Cipher::New( 
+	    config.cipherIface, config.keySize );
+    if(!cipher)
     {
 	cout << autosprintf(_("Unable to find specified cipher \"%s\"\n"),
 		config.cipherIface.name().c_str());
@@ -574,7 +635,7 @@ static int do_chpasswd( bool useStdin, int argc, char **argv )
     // ask for existing password
     cout << _("Enter current Encfs password\n");
     CipherKey userKey = getUserKey( cipher, useStdin );
-    if(userKey.isNull())
+    if(!userKey)
 	return EXIT_FAILURE;
 
     rAssert( (int)config.keyData.length() == cipher->encodedKeySize() );
@@ -585,7 +646,7 @@ static int do_chpasswd( bool useStdin, int argc, char **argv )
     CipherKey volumeKey = cipher->readKey(
 	    (unsigned char*)config.keyData.data(), userKey );
 
-    if(volumeKey.isNull())
+    if(!volumeKey)
     {
 	cout << _("Invalid password\n");
 	return EXIT_FAILURE;
@@ -601,7 +662,7 @@ static int do_chpasswd( bool useStdin, int argc, char **argv )
 
     // re-encode the volume key using the new user key and write it out..
     int result = EXIT_FAILURE;
-    if(!userKey.isNull())
+    if(userKey)
     {
 	int encodedKeySize = cipher->encodedKeySize();
 	unsigned char *keyBuf = new unsigned char[ encodedKeySize ];
@@ -686,7 +747,7 @@ int main(int argc, char **argv)
 
 	if(commands[offset].name == 0)
 	{
-	    cerr << autosprintf(_("invalid command: \"%s\""), argv[1]) << endl;
+	    cerr << autosprintf(_("invalid command: \"%s\""), argv[1]) << "\n";
 	} else
 	{
 	    if((argc-2 < commands[offset].minOptions) || 
@@ -694,7 +755,7 @@ int main(int argc, char **argv)
 	    {
 		cerr << autosprintf(
 			_("Incorrect number of arguments for command \"%s\""), 
-			argv[1]) << endl;
+			argv[1]) << "\n";
 	    } else
 		return (*commands[offset].func)( argc-1, argv+1 );
 	}
