@@ -29,29 +29,32 @@
 #include <unistd.h>
 #include <sys/time.h>
 #include <errno.h>
+#include <string.h>
 
 #include <getopt.h>
 
+#include <boost/scoped_ptr.hpp>
+#include <boost/shared_ptr.hpp>
+
 #include <rlog/rlog.h>
 #include <rlog/Error.h>
 #include <rlog/RLogChannel.h>
 #include <rlog/SyslogNode.h>
 #include <rlog/StdioNode.h>
 
-#include "Config.h"
+#include "ConfigReader.h"
 #include "Interface.h"
 #include "MemoryPool.h"
 #include "FileUtils.h"
 #include "DirNode.h"
+#include "Context.h"
 
-#ifdef HAVE_SSL
-#define NO_DES
-#include <openssl/ssl.h>
-#include <openssl/rand.h>
-#ifndef OPENSSL_NO_ENGINE
-#include <openssl/engine.h>
-#endif
-#endif
+#include "openssl.h"
+
+// Fuse version >= 26 requires another argument to fuse_unmount, which we
+// don't have.  So use the backward compatible call instead..
+extern "C" void fuse_unmount_compat22(const char *mountpoint);
+#    define fuse_unmount fuse_unmount_compat22
 
 #include <locale.h>
 
@@ -68,6 +71,8 @@ using namespace std;
 using namespace rlog;
 using namespace rel;
 using namespace gnu;
+using boost::shared_ptr;
+using boost::scoped_ptr;
 
 // Maximum number of arguments that we're going to pass on to fuse.  Doesn't
 // affect how many arguments we can handle, just how many we can pass on..
@@ -82,7 +87,7 @@ struct EncFS_Args
     const char *fuseArgv[MaxFuseArgs];
     int fuseArgc;
 
-    EncFS_Opts opts;
+    shared_ptr<EncFS_Opts> opts;
 
     // for debugging
     // In case someone sends me a log dump, I want to know how what options are
@@ -95,22 +100,23 @@ struct EncFS_Args
 	ss << (isThreaded ? "(threaded) " : "(UP) ");
 	if(idleTimeout > 0)
 	    ss << "(timeout " << idleTimeout << ") ";
-	if(opts.checkKey) ss << "(keyCheck) ";
-	if(opts.forceDecode) ss << "(forceDecode) ";
-	if(opts.ownerCreate) ss << "(ownerCreate) ";
-	if(opts.useStdin) ss << "(useStdin) ";
+	if(opts->checkKey) ss << "(keyCheck) ";
+	if(opts->forceDecode) ss << "(forceDecode) ";
+	if(opts->ownerCreate) ss << "(ownerCreate) ";
+	if(opts->useStdin) ss << "(useStdin) ";
+	if(opts->reverseEncryption) ss << "(reverseEncryption) ";
+	if(opts->mountOnDemand) ss << "(mountOnDemand) ";
 	for(int i=0; i<fuseArgc; ++i)
 	    ss << fuseArgv[i] << ' ';
 
 	return ss.str();
     }
-};
 
-/* Yeah, we can only deal with one filesystem at a time... */
-// This is a limitation of encfs's fuse connection layer, not encfs.
-DirNode *FSRoot = NULL;
-bool gPublicFilesystem = false;
-EncFS_Args *FSArgs = NULL;
+    EncFS_Args()
+	: opts( new EncFS_Opts() )
+    {
+    }
+};
 
 static int oldStderr = STDERR_FILENO;
 
@@ -138,6 +144,7 @@ void usage(const char *name)
 	                  "\t\t\t(for filesystems using MAC block headers)\n")
 	<< _("  --public\t\t"   "act as a typical multi-user filesystem\n"
 	                  "\t\t\t(encfs must be run as root)\n")
+	<< _("  --reverse\t\t"  "reverse encryption\n")
 
 	// xgroup(usage)
 	<< _("  --extpass=program\tUse external program for password prompt\n"
@@ -159,7 +166,7 @@ void FuseUsage()
 
     int argc = 2;
     const char *argv[] = {"...", "-h"};
-    fuse_main( argc, const_cast<char**>(argv), (fuse_operations*)NULL);
+    fuse_main( argc, const_cast<char**>(argv), (fuse_operations*)NULL, NULL);
 }
 
 #define PUSHARG(ARG) \
@@ -176,25 +183,21 @@ string slashTerminate( const string &src )
 }
 
 static 
-bool processArgs(int argc, char *argv[], EncFS_Args *out)
+bool processArgs(int argc, char *argv[], const shared_ptr<EncFS_Args> &out)
 {
     // set defaults
     out->isDaemon = true;
-#if defined(HAVE_ATOMIC_GCC) || defined(HAVE_ATOMIC_GCC_PRIVATE)
     out->isThreaded = true;
-#else
-    // no atomic update methods -- safest to stick to single-threaded mode..
-    out->isThreaded = false;
-#endif
     out->isVerbose = false;
     out->idleTimeout = 0;
     out->fuseArgc = 0;
-    out->opts.idleTracking = false;
-    out->opts.checkKey = true;
-    out->opts.forceDecode = false;
-    out->opts.ownerCreate = false;
-    out->opts.useStdin = false;
-    
+    out->opts->idleTracking = false;
+    out->opts->checkKey = true;
+    out->opts->forceDecode = false;
+    out->opts->ownerCreate = false;
+    out->opts->useStdin = false;
+    out->opts->reverseEncryption = false;
+ 
     bool useDefaultFlags = true;
 
     // pass executable name through
@@ -215,12 +218,14 @@ bool processArgs(int argc, char *argv[], EncFS_Args *out)
 	{"idle", 1, 0, 'i'}, // idle timeout
 	{"anykey", 0, 0, 'k'}, // skip key checks
 	{"no-default-flags", 0, 0, 'N'}, // don't use default fuse flags
+	{"ondemand", 0, 0, 'm'}, // mount on-demand
 	{"public", 0, 0, 'P'}, // public mode
 	{"extpass", 1, 0, 'p'}, // external password program
 	// {"single-thread", 0, 0, 's'}, // single-threaded mode
-	{"stdinpass", 1, 0, 'S'}, // read password from stdin
+	{"stdinpass", 0, 0, 'S'}, // read password from stdin
 	{"verbose", 0, 0, 'v'}, // verbose mode
 	{"version", 0, 0, 'V'}, //version
+	{"reverse", 0, 0, 'r'}, // reverse encryption
 	{0,0,0,0}
     };
 
@@ -233,8 +238,10 @@ bool processArgs(int argc, char *argv[], EncFS_Args *out)
 	// 'v' : verbose mode (same as --verbose)
 	// 'd' : fuse debug mode (same as --fusedebug)
 	// 'i' : idle-timeout, takes argument
+	// 'm' : mount-on-demand
 	// 'S' : password from stdin
-	int res = getopt_long( argc, argv, "HsSfvdi:",
+	// 'o' : arguments meant for fuse
+	int res = getopt_long( argc, argv, "HsSfvdmi:o:",
 		long_options, &option_index);
 
 	if(res == -1)
@@ -246,7 +253,7 @@ bool processArgs(int argc, char *argv[], EncFS_Args *out)
 	    out->isThreaded = false;
 	    break;
 	case 'S':
-	    out->opts.useStdin = true;
+	    out->opts->useStdin = true;
 	    break;
 	case 'f':
 	    out->isDaemon = false;
@@ -260,33 +267,37 @@ bool processArgs(int argc, char *argv[], EncFS_Args *out)
 	    PUSHARG("-d");
 	    break;
 	case 'i':
-#if FUSE_MAJOR_VERSION == 2 && FUSE_MINOR_VERSION == 2
-	    cerr << "Sorry, idle monitoring not enabled. "
-		"FUSE 2.3 or newer required.\n" 
-		<< endl;
-#else
 	    out->idleTimeout = strtol( optarg, (char**)NULL, 10);
-	    out->opts.idleTracking = true;
-#endif
+	    out->opts->idleTracking = true;
 	    break;
 	case 'k':
-	    out->opts.checkKey = false;
+	    out->opts->checkKey = false;
 	    break;
 	case 'D':
-	    out->opts.forceDecode = true;
+	    out->opts->forceDecode = true;
+	    break;
+	case 'r':
+	    out->opts->reverseEncryption = true;	    
+	    break;
+	case 'm':
+	    out->opts->mountOnDemand = true;
 	    break;
 	case 'N':
 	    useDefaultFlags = false;
 	    break;
+	case 'o':
+	    PUSHARG("-o");
+	    PUSHARG( optarg );
+	    break;
 	case 'p':
-	    out->opts.passwordProgram.assign( optarg );
+	    out->opts->passwordProgram.assign( optarg );
 	    break;
 	case 'P':
 	    if(geteuid() != 0)
 		rWarning(_("option '--public' ignored for non-root user"));
 	    else
 	    {
-		out->opts.ownerCreate = true;
+		out->opts->ownerCreate = true;
 		// add 'allow_other' option
 		// add 'default_permissions' option (default)
 		PUSHARG("-o");
@@ -329,7 +340,7 @@ bool processArgs(int argc, char *argv[], EncFS_Args *out)
     // the mount point.
     if(optind+2 <= argc)
     {
-	out->opts.rootDir = slashTerminate( argv[optind++] );
+	out->opts->rootDir = slashTerminate( argv[optind++] );
 	out->mountPoint = argv[optind++];
     } else
     {
@@ -354,7 +365,7 @@ bool processArgs(int argc, char *argv[], EncFS_Args *out)
     // sanity check
     if(out->isDaemon && 
 	    (!isAbsolutePath( out->mountPoint.c_str() ) ||
-	    !isAbsolutePath( out->opts.rootDir.c_str() ) ) 
+	    !isAbsolutePath( out->opts->rootDir.c_str() ) ) 
       )
     {
 	cerr << 
@@ -369,7 +380,7 @@ bool processArgs(int argc, char *argv[], EncFS_Args *out)
     {
 	string testMountPoint = slashTerminate( out->mountPoint );
 	string testRootDir = 
-	    out->opts.rootDir.substr(0, testMountPoint.length());
+	    out->opts->rootDir.substr(0, testMountPoint.length());
 
 	if( testMountPoint == testRootDir )
 	{
@@ -381,9 +392,18 @@ bool processArgs(int argc, char *argv[], EncFS_Args *out)
 	}
     }
 
+    if(out->opts->mountOnDemand && out->opts->passwordProgram.empty())
+    {
+	cerr << 
+	    // xgroup(usage)
+	    _("Must set password program when using mount-on-demand")
+	    << endl;
+	return false;
+    }
+
     // check that the directories exist, or that we can create them..
-    if(!isDirectory( out->opts.rootDir.c_str() ) && 
-	    !userAllowMkdir( out->opts.rootDir.c_str() ,0700))
+    if(!isDirectory( out->opts->rootDir.c_str() ) && 
+	    !userAllowMkdir( out->opts->rootDir.c_str() ,0700))
     {
 	rWarning(_("Unable to locate root directory, aborting."));
 	return false;
@@ -401,141 +421,21 @@ bool processArgs(int argc, char *argv[], EncFS_Args *out)
     return true;
 }
 
-/*
-    I think I nabbed this stuff from a test program from the OpenSSL
-    distribution..
-*/
-#ifdef HAVE_SSL
-unsigned long pthreads_thread_id()
+static void * idleMonitor(void *);
+
+void *encfs_init(fuse_conn_info *conn)
 {
-    return (unsigned long)pthread_self();
-}
+    EncFS_Context *ctx = (EncFS_Context*)fuse_get_context()->private_data;
 
-static pthread_mutex_t *crypto_locks = NULL;
-void pthreads_locking_callback( int mode, int n,
-	const char *caller_file, int caller_line )
-{
-    (void)caller_file;
-    (void)caller_line;
-
-    if(!crypto_locks)
-    {
-	rDebug("Allocating %i locks for OpenSSL", CRYPTO_num_locks() );
-	crypto_locks = new pthread_mutex_t[ CRYPTO_num_locks() ];
-	for(int i=0; i<CRYPTO_num_locks(); ++i)
-	    pthread_mutex_init( crypto_locks+i, 0 );
-    }
-
-    if(mode & CRYPTO_LOCK)
-    {
-	pthread_mutex_lock( crypto_locks + n );
-    } else
-    {
-	pthread_mutex_unlock( crypto_locks + n );
-    }
-}
-
-void pthreads_locking_cleanup()
-{
-    if(crypto_locks)
-    {
-	for(int i=0; i<CRYPTO_num_locks(); ++i)
-	    pthread_mutex_destroy( crypto_locks+i );
-	delete[] crypto_locks;
-	crypto_locks = NULL;
-    }
-}
-#endif
-
-struct EncfsFSContext
-{
-    EncFS_Args *args;
-
-    // for idle monitor
-    bool running;
-    pthread_t monitorThread;
-    pthread_cond_t wakeupCond;
-    pthread_mutex_t wakeupMutex;
-};
-
-/*
-    Idle monitoring thread.  This is only used when idle monitoring is enabled.
-    It will cause the filesystem to be automatically unmounted (causing us to
-    commit suicide) if the filesystem stays idle too long.  Idle time is only
-    checked if there are no open files, as I don't want to risk problems by
-    having the filesystem unmounted from underneath open files!
-*/
-const int MinActivityCheckInterval = 10;
-
-static
-void * idleMonitor(void *_arg)
-{
-    EncfsFSContext *ctx = (EncfsFSContext*)_arg;
-    EncFS_Args *arg = ctx->args;
-
-    int timeoutSeconds = 60 * arg->idleTimeout;
-
-    pthread_mutex_lock( &ctx->wakeupMutex );
-    
-    while(ctx->running)
-    {
-	int nextCheck = MinActivityCheckInterval;
-
-	int idleSeconds = FSRoot->idleSeconds();
-	if(idleSeconds >= timeoutSeconds)
-	{
-	    if( !FSRoot->hasOpenFiles() )
-	    {
-		// Time to unmount!
-		// xgroup(diag)
-		rWarning(_("Unmounting filesystem %s due to inactivity"),
-			arg->mountPoint.c_str());
-		fuse_unmount( arg->mountPoint.c_str() );
-		// wait for main thread to wake us up
-		pthread_cond_wait( &ctx->wakeupCond, &ctx->wakeupMutex );
-		break;
-	    } else
-	    {
-		rDebug("open files: \n%s", FSRoot->openFileList().c_str());
-	    }
-	} else
-	{
-	    // don't need to check again until it could possibly timeout
-	    nextCheck = MAX(timeoutSeconds - idleSeconds,
-		            MinActivityCheckInterval);
-	    rDebug("checking idle time again in %i seconds", nextCheck);
-	}
-
-	struct timeval currentTime;
-	gettimeofday( &currentTime, 0 );
-	struct timespec wakeupTime;
-	wakeupTime.tv_sec = currentTime.tv_sec + nextCheck;
-	wakeupTime.tv_nsec = currentTime.tv_usec * 1000;
-	pthread_cond_timedwait( &ctx->wakeupCond, 
-		&ctx->wakeupMutex, &wakeupTime );
-    }
-    
-    pthread_mutex_unlock( &ctx->wakeupMutex );
-
-    rDebug("Idle monitoring thread exiting");
-
-    return 0;
-}
-
-void *encfs_init()
-{
-    EncfsFSContext *ctx = new EncfsFSContext;
-
-    ctx->args = FSArgs;
+    // set fuse connection options
+    conn->async_read = true;
 
     // if an idle timeout is specified, then setup a thread to monitor the
     // filesystem.
-    if(FSArgs->idleTimeout > 0)
+    if(ctx->args->idleTimeout > 0)
     {
 	rDebug("starting idle monitoring thread");
 	ctx->running = true;
-	pthread_cond_init( &ctx->wakeupCond, 0 );
-	pthread_mutex_init( &ctx->wakeupMutex, 0 );
 
 	int res = pthread_create( &ctx->monitorThread, 0, idleMonitor, 
 		(void*)ctx );
@@ -546,8 +446,9 @@ void *encfs_init()
 	}
     }
 
-    if(oldStderr >= 0)
+    if(ctx->args->isDaemon && oldStderr >= 0)
     {
+	rInfo("Closing stderr");
 	close(oldStderr);
 	oldStderr = -1;
     }
@@ -557,7 +458,7 @@ void *encfs_init()
  
 void encfs_destroy( void *_ctx )
 {
-    EncfsFSContext *ctx = (EncfsFSContext*)_ctx;
+    EncFS_Context *ctx = (EncFS_Context*)_ctx;
     if(ctx->args->idleTimeout > 0)
     {
 	ctx->running = false;
@@ -584,27 +485,9 @@ int main(int argc, char *argv[])
     textdomain( PACKAGE );
 #endif
 
-#ifdef HAVE_SSL
-    // initialize the SSL library
-    SSL_load_error_strings();
-    SSL_library_init();
-
-    unsigned int randSeed = 0;
-    RAND_bytes( (unsigned char*)&randSeed, sizeof(randSeed) );
-    srand( randSeed );
-
-#ifndef OPENSSL_NO_ENGINE
-    // initialize hardware crypto engine support
-    ENGINE_load_builtin_engines();
-    ENGINE_register_all_ciphers();
-    ENGINE_register_all_digests();
-    ENGINE_register_all_RAND();
-#endif // NO_ENGINE
-#endif
-
     // log to stderr by default..
-    StdioNode *slog = new StdioNode( STDERR_FILENO );
-    SyslogNode *logNode = NULL;
+    scoped_ptr<StdioNode> slog( new StdioNode( STDERR_FILENO ) );
+    scoped_ptr<SyslogNode> logNode;
 
     // show error and warning output
     slog->subscribeTo( GetGlobalChannel("error") );
@@ -612,7 +495,7 @@ int main(int argc, char *argv[])
 
     // anything that comes from the user should be considered tainted until
     // we've processed it and only allowed through what we support.
-    EncFS_Args *encfsArgs = new EncFS_Args;
+    shared_ptr<EncFS_Args> encfsArgs( new EncFS_Args );
     for(int i=0; i<MaxFuseArgs; ++i)
 	encfsArgs->fuseArgv[i] = NULL; // libfuse expects null args..
 
@@ -629,17 +512,7 @@ int main(int argc, char *argv[])
 	slog->subscribeTo( GetGlobalChannel("debug") );
     }
   
-#ifdef HAVE_SSL
-    if(encfsArgs->isThreaded)
-    {
-	// provide locking functions to OpenSSL since we'll be running with
-	// threads accessing openssl in parallel.
-	CRYPTO_set_id_callback( pthreads_thread_id );
-	CRYPTO_set_locking_callback( pthreads_locking_callback );
-    }
-#endif
-
-    rDebug("Root directory: %s", encfsArgs->opts.rootDir.c_str());
+    rDebug("Root directory: %s", encfsArgs->opts->rootDir.c_str());
     rDebug("Fuse arguments: %s", encfsArgs->toString().c_str());
 	
     fuse_operations encfs_oper;
@@ -666,7 +539,7 @@ int main(int argc, char *argv[])
     encfs_oper.read = encfs_read;
     encfs_oper.write = encfs_write;
     encfs_oper.statfs = encfs_statfs;
-    //encfs_oper.flush = encfs_flush;
+    encfs_oper.flush = encfs_flush;
     encfs_oper.release = encfs_release;
     encfs_oper.fsync = encfs_fsync;
 #ifdef HAVE_XATTR
@@ -683,19 +556,27 @@ int main(int argc, char *argv[])
     encfs_oper.destroy = encfs_destroy;
     //encfs_oper.access = encfs_access;
     //encfs_oper.create = encfs_create;
-    //encfs_oper.ftruncate = encfs_ftruncate;
-    //encfs_oper.fgetattr = encfs_fgetattr;
+    encfs_oper.ftruncate = encfs_ftruncate;
+    encfs_oper.fgetattr = encfs_fgetattr;
+    encfs_oper.utimens = encfs_utimens;
+    //encfs_oper.bmap = encfs_bmap;
 
-    gPublicFilesystem = encfsArgs->opts.ownerCreate;
-    RootPtr rootInfo = initFS( &encfsArgs->opts );
+    openssl_init( encfsArgs->isThreaded );
+
+    // context is not a smart pointer because it will live for the life of
+    // the filesystem.
+    EncFS_Context *ctx = new EncFS_Context;
+    ctx->publicFilesystem = encfsArgs->opts->ownerCreate;
+    RootPtr rootInfo = initFS( ctx, encfsArgs->opts );
     
     int returnCode = EXIT_FAILURE;
 
-    if( !rootInfo.isNull() )
+    if( rootInfo )
     {
 	// set the globally visible root directory node
-	FSRoot = rootInfo->root.get();
-	FSArgs = encfsArgs;
+	ctx->setRoot( rootInfo->root );
+	ctx->args = encfsArgs;
+	ctx->opts = encfsArgs->opts;
 	    
 	if(encfsArgs->isThreaded == false && encfsArgs->idleTimeout > 0)
 	{
@@ -713,13 +594,12 @@ int main(int argc, char *argv[])
 	if(encfsArgs->isDaemon)
 	{
 	    // switch to logging just warning and error messages via syslog
-	    logNode = new SyslogNode( "encfs" );
+	    logNode.reset( new SyslogNode( "encfs" ) );
 	    logNode->subscribeTo( GetGlobalChannel("warning") );
 	    logNode->subscribeTo( GetGlobalChannel("error") );
 
 	    // disable stderr reporting..
-	    delete slog;
-	    slog = NULL;
+	    slog.reset();
 
 	    // keep around a pointer just in case we end up needing it to
 	    // report a fatal condition later (fuse_main exits unexpectedly)...
@@ -738,7 +618,7 @@ int main(int argc, char *argv[])
 	    // fuse_main returns an error code in newer versions of fuse..
 	    int res = fuse_main( encfsArgs->fuseArgc, 
 		    const_cast<char**>(encfsArgs->fuseArgv), 
-		    &encfs_oper);
+		    &encfs_oper, (void*)ctx);
 	    
 	    time( &endTime );
 
@@ -769,15 +649,94 @@ int main(int argc, char *argv[])
 
     // cleanup so that we can check for leaked resources..
     rootInfo.reset();
-    FSRoot = NULL;
+    ctx->setRoot( shared_ptr<DirNode>() );
 
     MemoryPool::destroyAll();
-#ifdef HAVE_SSL
-    if(encfsArgs->isThreaded)
-	pthreads_locking_cleanup();
-#endif
-    delete logNode;
-    delete encfsArgs;
+    openssl_shutdown( encfsArgs->isThreaded );
 
     return returnCode;
 }
+
+/*
+    Idle monitoring thread.  This is only used when idle monitoring is enabled.
+    It will cause the filesystem to be automatically unmounted (causing us to
+    commit suicide) if the filesystem stays idle too long.  Idle time is only
+    checked if there are no open files, as I don't want to risk problems by
+    having the filesystem unmounted from underneath open files!
+*/
+const int ActivityCheckInterval = 10;
+static bool unmountFS(EncFS_Context *ctx);
+
+static
+void * idleMonitor(void *_arg)
+{
+    EncFS_Context *ctx = (EncFS_Context*)_arg;
+    shared_ptr<EncFS_Args> arg = ctx->args;
+
+    const int timeoutCycles = 60 * arg->idleTimeout / ActivityCheckInterval;
+    int idleCycles = 0;
+
+    pthread_mutex_lock( &ctx->wakeupMutex );
+    
+    while(ctx->running)
+    {
+	int usage = ctx->getAndResetUsageCounter();
+
+	if(usage == 0 && ctx->isMounted())
+	    ++idleCycles;
+	else
+	    idleCycles = 0;
+	
+	if(idleCycles >= timeoutCycles)
+	{
+	    int openCount = ctx->openFileCount();
+	    if( openCount == 0 && unmountFS( ctx ) )
+	    {
+		// wait for main thread to wake us up
+		pthread_cond_wait( &ctx->wakeupCond, &ctx->wakeupMutex );
+		break;
+	    }
+		
+	    rDebug("num open files: %i", openCount );
+	}
+	    
+	rDebug("idle cycle count: %i, timeout after %i", idleCycles,
+		timeoutCycles);
+
+	struct timeval currentTime;
+	gettimeofday( &currentTime, 0 );
+	struct timespec wakeupTime;
+	wakeupTime.tv_sec = currentTime.tv_sec + ActivityCheckInterval;
+	wakeupTime.tv_nsec = currentTime.tv_usec * 1000;
+	pthread_cond_timedwait( &ctx->wakeupCond, 
+		&ctx->wakeupMutex, &wakeupTime );
+    }
+    
+    pthread_mutex_unlock( &ctx->wakeupMutex );
+
+    rDebug("Idle monitoring thread exiting");
+
+    return 0;
+}
+
+static bool unmountFS(EncFS_Context *ctx)
+{
+    shared_ptr<EncFS_Args> arg = ctx->args;
+    if( arg->opts->mountOnDemand )
+    {
+	rDebug("Detaching filesystem %s due to inactivity",
+		arg->mountPoint.c_str());
+
+	ctx->setRoot( shared_ptr<DirNode>() );
+	return false;
+    } else
+    {
+	// Time to unmount!
+	// xgroup(diag)
+	rWarning(_("Unmounting filesystem %s due to inactivity"),
+		arg->mountPoint.c_str());
+	fuse_unmount( arg->mountPoint.c_str() );
+	return true;
+    }
+}
+
