@@ -29,7 +29,7 @@
 #include "MemoryPool.h"
 #include "Interface.h"
 #include "FileUtils.h"
-#include "Config.h"
+#include "ConfigReader.h"
 #include "StreamNameIO.h"
 #include "BlockNameIO.h"
 #include "NullNameIO.h"
@@ -52,11 +52,12 @@ using namespace std;
 using namespace rel;
 using namespace rlog;
 
+using boost::shared_ptr;
     
 const int FSBlockSize = 256;
 
 static
-int checkErrorPropogation( const Ptr<Cipher> &cipher,
+int checkErrorPropogation( const shared_ptr<Cipher> &cipher,
 	int size, int byteToChange, const CipherKey &key )
 {
     MemBlock orig = MemoryPool::allocate(size);
@@ -163,7 +164,7 @@ bool testNameCoding( DirNode &dirNode, bool verbose )
     return true;
 }
 
-bool runTests(const Ptr<Cipher> &cipher, bool verbose)
+bool runTests(const shared_ptr<Cipher> &cipher, bool verbose)
 {
     // create a random key
     if(verbose)
@@ -179,7 +180,7 @@ bool runTests(const Ptr<Cipher> &cipher, bool verbose)
 
 	cipher->writeKey( key, keyBuf, encodingKey );
 	CipherKey key2 = cipher->readKey( keyBuf, encodingKey );
-	if(key2.isNull())
+	if(!key2)
 	{
 	    if(verbose)
 		cerr << "   FAILED (decode error)\n";
@@ -217,7 +218,7 @@ bool runTests(const Ptr<Cipher> &cipher, bool verbose)
 	cfg.keyData.assign( (char*)keyBuf, encodedKeySize );
 
 	// save config
-	Config cfgRdrOut;
+	ConfigReader cfgRdrOut;
 	cfgRdrOut["cipher"] << cfg.cipherIface;
 	cfgRdrOut["keySize"] << cfg.keySize;
 	cfgRdrOut["blockSize"] << cfg.blockSize;
@@ -227,7 +228,7 @@ bool runTests(const Ptr<Cipher> &cipher, bool verbose)
 	ConfigVar storage = cfgRdrOut.toVar();
 
 	// read back in and check everything..
-	Config cfgRdrIn;
+	ConfigReader cfgRdrIn;
 	cfgRdrIn.loadFromVar( storage );
 
 	EncFSConfig cfg2;
@@ -246,7 +247,7 @@ bool runTests(const Ptr<Cipher> &cipher, bool verbose)
 
 	CipherKey key2 = cipher->readKey( (unsigned char*)cfg2.keyData.data(), 
 		encodingKey );
-	if(key2.isNull())
+	if(!key2)
 	{
 	    if(verbose)
 		cerr << "   FAILED (decode error)\n";
@@ -267,7 +268,7 @@ bool runTests(const Ptr<Cipher> &cipher, bool verbose)
 	delete[] keyBuf;
     }
 
-    Ptr<DirNode::Config> dnConfig( new DirNode::Config );
+    shared_ptr<DirNode::Config> dnConfig( new DirNode::Config );
     dnConfig->cipher = cipher;
     dnConfig->key = key;
     dnConfig->blockSize = FSBlockSize;
@@ -279,11 +280,11 @@ bool runTests(const Ptr<Cipher> &cipher, bool verbose)
 	dnConfig->blockMACBytes = 0;
 	dnConfig->blockMACRandBytes = 0;
 	dnConfig->uniqueIV = false;
-	dnConfig->nameCoding = Ptr<NameIO>( new StreamNameIO( 
+	dnConfig->nameCoding = shared_ptr<NameIO>( new StreamNameIO( 
 		    StreamNameIO::CurrentInterface(), cipher, key ) );
 	dnConfig->nameCoding->setChainedNameIV( true );
 
-	DirNode dirNode( TEST_ROOTDIR, dnConfig );
+	DirNode dirNode( NULL, TEST_ROOTDIR, dnConfig );
 
 	if(!testNameCoding( dirNode, verbose ))
 	    return false;
@@ -296,12 +297,12 @@ bool runTests(const Ptr<Cipher> &cipher, bool verbose)
 	dnConfig->blockMACBytes = 0;
 	dnConfig->blockMACRandBytes = 0;
 	dnConfig->uniqueIV = false;
-	dnConfig->nameCoding = Ptr<NameIO>( new BlockNameIO( 
+	dnConfig->nameCoding = shared_ptr<NameIO>( new BlockNameIO( 
 		    BlockNameIO::CurrentInterface(), cipher, key,
 		    cipher->cipherBlockSize() ) );
 	dnConfig->nameCoding->setChainedNameIV( true );
 	
-	DirNode dirNode( TEST_ROOTDIR, dnConfig );
+	DirNode dirNode( NULL, TEST_ROOTDIR, dnConfig );
 
 	if(!testNameCoding( dirNode, verbose ))
 	    return false;
@@ -316,11 +317,11 @@ bool runTests(const Ptr<Cipher> &cipher, bool verbose)
 	    dnConfig->blockMACRandBytes = 0;
 	    dnConfig->uniqueIV = false;
 	    dnConfig->nameCoding = 
-		Ptr<NameIO>( new StreamNameIO( 
+		shared_ptr<NameIO>( new StreamNameIO( 
 			    StreamNameIO::CurrentInterface(), cipher, key ) );
 	    dnConfig->nameCoding->setChainedNameIV( false );
 
-	    DirNode dirNode( TEST_ROOTDIR, dnConfig );
+	    DirNode dirNode( NULL, TEST_ROOTDIR, dnConfig );
 
 	    if(!testNameCoding( dirNode, verbose ))
 	       	return false;
@@ -332,12 +333,12 @@ bool runTests(const Ptr<Cipher> &cipher, bool verbose)
 	    dnConfig->blockMACBytes = 0;
 	    dnConfig->blockMACRandBytes = 0;
 	    dnConfig->uniqueIV = false;
-	    dnConfig->nameCoding = Ptr<NameIO>( new BlockNameIO( 
+	    dnConfig->nameCoding = shared_ptr<NameIO>( new BlockNameIO( 
 			BlockNameIO::CurrentInterface(), cipher, key,
 			cipher->cipherBlockSize() ) );
 	    dnConfig->nameCoding->setChainedNameIV( false );
 
-	    DirNode dirNode( TEST_ROOTDIR, dnConfig );
+	    DirNode dirNode( NULL, TEST_ROOTDIR, dnConfig );
 
 	    if(!testNameCoding( dirNode, verbose ))
 	       	return false;
@@ -499,8 +500,8 @@ int main(int argc, char *argv[])
 	    cerr << it->name << ", key length " << keySize
 		<< ", block size " << blockSize << ":  ";
 
-	    Ptr<Cipher> cipher = Cipher::New( it->name, keySize );
-	    if(cipher.isNull())
+	    shared_ptr<Cipher> cipher = Cipher::New( it->name, keySize );
+	    if(!cipher)
 	    {
 		cerr << "FAILED TO CREATE\n";
 	    } else
@@ -520,8 +521,8 @@ int main(int argc, char *argv[])
     }
 
     // run one test with verbose output too..
-    Ptr<Cipher> cipher = Cipher::New("AES", 192);
-    if(cipher.isNull())
+    shared_ptr<Cipher> cipher = Cipher::New("AES", 192);
+    if(!cipher)
     {
 	cerr << "\nNo AES cipher found, skipping verbose test.\n";
     } else
