@@ -1,10 +1,7 @@
 #!/bin/sh
 
-echo Changing file permissions...
-chmod -R u+w .
-
 echo Creating autoconf scripts...
-autoreconf
+autoreconf -i
 
 echo Configuring...
 ./configure
